#Qian 8/30/2012 paired data show patient ID instead of array ID (such as Perou dataset).
# 8/10/08. Tilman: First version of function CCheatmap to produce a clustered heatmap of expression values
#                  with arrays grouped by class, for use in class comparison output. Function is based on
#                  Jacob's TCHeatmap function, written for his time-course analysis plugin.
CCHeatmap <- function(xdata, geneidlabel, classid, arrayid, ExpId, pairedid, isSingleChannel = FALSE, NumColors, MinColor, MidColor, MaxColor, #MissColor,
                      dividers = NULL, outputdir, outpfilepng, outpfilehtml, paired, ArrayToolsPath){
  # Input:
  #   xdata: expression data; rows: genes in increasing P-value, columns: arrays. Should have at least 5 genes. Missing values must be coded NA.
  #   geneidlabel: label for the genenames to be displayed in the heatmap.
  #   classid: the class of the arrays in the columns of xdata.
  #   arrayid: label for the arrays to be displayed in the heatmap.
  #   ExpId: lables for the arrays used for matching the experiments order.
  #   isSingleChannel: Boolean - whether or not the expression data is single channel.
  #   NumColors: Number of colors
  #   MinColor: Color for negative data values
  #   MidColor: Color for data values around zero
  #   MaxColor: Color for positive data values
  #   MissColor: Color for missing values *** NOT YET IMPLEMENTED - function automatically displays missing values as blank (white)
  #   dividers: Colour of vertical lines visually dividing the classes/groups in the heatmap. NULL means no dividers.
  #   outputdir: Pathname of the directory containing the outputfiles.
  #   outpfilepng: Filename of the png file containing the sorted heatmap.
  #   outpfilehtml: Filename of the HTML file in which the heatmap can be viewed.
  #   paired: indicator for paired calculation. #Qian 03/30/09
  # OUTPUT: The png-file outpfilepng containing the sorted heatmap and the HTML file outpfilehtml
  #         in which the sorted heatmap can be viewed.
  cat("Preparing the heatmap ...", "", sep="\n")
  ord <- match(ExpId, arrayid)                  #the array order will be the same as in the expressin table.
  classid <- classid[ord]                       #--> classes in alphabetical order
  arrayid <- arrayid[ord]
  #Qian 8/30/2012 paired data show patient ID instead of array ID (such as Perou dataset).
  if(paired) {arrayid<-pairedid[ord]}
  
  xdata[xdata < -999] <- NA                     # MLI modified.
  #  if(isSingleChannel) xdata <- sweep(xdata,1,apply(xdata,1,median,na.rm=TRUE)) #--> centers expression data by subtracting median expression #Qian modified on 1/15/2009
  #    level for each gene (across all classes)
  
  #Qian 12/14/09 for only one significant gene.
  if(dim(xdata)[1] == 1) {
    clustered <- 1
  } else {
    clustered <- hclust(dist(xdata,method="euclidian"),method="average")$order
  }
  xdata <- xdata[clustered,] #--> HARD CODED: clusters and orders genes using euclidian distance and average linkage
  
  geneidlabel <- geneidlabel[clustered] #--> ensures gene labels are sorted accordingly as per clustering
  
  #Qian 6/26/09
  #source(paste(ArrayToolsPath, "R/BRBHeatmap.R", sep="/"))
  BRBHeatmap(AnalysisName = "ClassComparison",
             pngfilename = paste(outputdir,outpfilepng,sep="/"),
             xdata = xdata,
             arrayid = arrayid,
             classid = classid,
             geneid = geneidlabel,
             ArrayToolsPath = ArrayToolsPath,
             isSingleChannel = isSingleChannel,
             paired = paired)
  #EndQian
  
  htmlfile <- paste(outputdir,outpfilehtml,sep="/")
  #Qian 03/30/09 for paired  
  if(paired) {
    cat("<html> <head><font size=4><b>Clustered heatmap of significantly expressed genes.</b></font></head><body>\n<p>\n",file=htmlfile)
  } else {
    cat("<html> <head><font size=4><b>Clustered heatmap of significantly expressed genes; arrays grouped by class.</b></font></head><body>\n<p>\n",file=htmlfile)
  }
  #EndQian
  cat("<img src = ",'\"',outpfilepng ,'\"', "><BR>\n", sep="", file=htmlfile, append=TRUE)
  cat("To modify the heatmap colors and threshold parameters, from the \"ArrayTools\" pull down menu select Utilities -> Preference and then edit the corresponding settings.\n", file=htmlfile, append=TRUE) #Qian 7/21/09
  cat("\n</p>\n</body></html>", file=htmlfile, append=TRUE)
}
