# From Input_to_output_function.R
# AMY MODIFIED THIS SECTION TO PRINT EXPRESSION DATA TABLE INTO SEPARATE FILE,
# INSTEAD OF APPENDING TABLE TO END OF MAIN OUTPUT RESULTS FILE.
CreateExpressionTable <- function(projectPath,
                                  AnalysisName,
                                  AnalysisType,
                                  WorkPath,
                                  ArrayToolsPath,
                                  nGenesInTable,
                                  HEAT.ERROR,
                                  heatmap.error.text,
                                  heatmap.error.flags,
                                  RunLFDR,
                                  classifiers,
                                  codeReg,
                                  paired
) {
	DataTableFileName <- paste(projectPath, "/Output/", AnalysisName, "/ExpressionTable.html", sep="")
	unlink(DataTableFileName)

	# Determine whether or not to write the expression data.
	if (AnalysisType == "SAM") {
		# Modified for possible error from running fortran program. Mingchung 5/19/2011
		if (file.exists(file.path(WorkPath, "sam_output.txt"))) {
			nsiggenes <- scan(paste(WorkPath,"/sam_output.txt", sep="/"), n=10, quiet=TRUE)[8:9]
			WriteExpressionData <- if (sum(nsiggenes) > 0) TRUE else FALSE
		} else {
			WriteExpressionData <- FALSE
		}	
	} else {
		WriteExpressionData <- (file.exists(paste(WorkPath, "/GeneExpData.txt", sep="")) &&
		  length(scan(paste(WorkPath, "/GeneExpData.txt", sep=""),nmax=1, quiet=TRUE)) > 0)
		# GeneExpData may contain extra genes' expression since Fortran will use
		# multiple options to run.
		if (AnalysisType == "ClassComparison" | 
			AnalysisType == "BlockClassComparison" |
			AnalysisType == "Survival" |
			AnalysisType == "Correlation") 
			WriteExpressionData <- WriteExpressionData & (nGenesInTable > 0) 
		if (AnalysisType == "ClassPrediction") 
			WriteExpressionData <- WriteExpressionData & (numDEGs > 0)
	}

	# Begin if-WriteExpressionData-loop 
	if (WriteExpressionData) {

		#    cat("<P></P><HR SIZE=5 WIDTH=\"100%\" NOSHADE><P></P><A HREF=ExpressionTable.html>View expression of significant genes.</A>" # Qian 4/9/09
		#         ,file=ExternalFileName, sep="\n", append=TRUE)

		cat("<HTML>"
			, paste("<script src='file:///", ArrayToolsPath, "/Fortran/sorttable.js'></script>", sep="") # Qian 5/29/09 sortable
			, "<BODY>"," ", file=DataTableFileName, sep="\n", append=FALSE)

		# Tilman added this to include link to CC heatmap 8/19/08. 
		# 'HEAT.ERROR', 'heatmap.error.text' and 'heatmap.error.flags' generated in 'GetComparisonResults.R'
		if(AnalysisType == "ClassComparison" | AnalysisType == "BlockClassComparison") { 
			if(HEAT.ERROR){
				outpfilehtml = paste(projectPath, "/Output/", AnalysisName, "/heatmap.html", sep="")
				cat("<P>UNABLE TO PRODUCE HEATMAP*<BR><BR>",file=outpfilehtml, sep="\n", append=FALSE)
				cat(paste("<P>*",heatmap.error.text[heatmap.error.flags],sep=""),file=outpfilehtml, sep="", append=TRUE)
				cat("<HTML>"
					, paste("<script src='file:///", ArrayToolsPath, "/Fortran/sorttable.js'></script>", sep="") # Qian 5/29/09 sortable
					, "<BODY>"," ", file=DataTableFileName, sep="\n", append=FALSE)
			} 
		}

		#    Experiments <- dget(paste(ArrayToolsPath, "/Fortran/experiments.rda", sep=""))
		load(paste(WorkPath, "/experiments.rda", sep="")) # use load/save method, Yong 12/6/06. Use WorkPath, 12/15/06
		Experiments <- experiments

		if (AnalysisType == "SAM") {
			ExpressionData.geneid <- index
			ExpressionData<-arr1[index,,drop = FALSE]
		} else if (AnalysisType == "ClassComparison" && RunLFDR) {
			 ExpressionData.geneid <- classifiers$Index
			 ExpressionData <- as.matrix(read.delim(file.path(WorkPath, "LR_data.txt"), skip=2, header=FALSE))
			 ExpressionData[ExpressionData < -100000] <- NA
			 if (codeReg) ExpressionData <- ExpressionData[ExpressionData[, ncol(ExpressionData)]==1, -ncol(ExpressionData)]
			 if (paired) ExpressionData <- ExpressionData[, 1:(ncol(ExpressionData)/2), drop = FALSE] - 
										   ExpressionData[, (ncol(ExpressionData)/2 + 1):ncol(ExpressionData), drop = FALSE] 		 		 
			 ExpressionData <- ExpressionData[ExpressionData.geneid, , drop = FALSE]
			 ExpressionData[is.na(ExpressionData)] <- -9999999
		} else {
			ExpressionData.raw <- read.table(paste(WorkPath, "/GeneExpData.txt", sep=""))
			if (!exists("ngenesExp")) ngenesExp <- nrow(ExpressionData.raw)
			ExpressionData.geneid <- ExpressionData.raw[1:ngenesExp,1]
			ExpressionData        <- ExpressionData.raw[1:ngenesExp,-1]
			ExpressionData        <- as.matrix(ExpressionData)
		}
		ExpressionData <- ifelse(ExpressionData < -100000 | ExpressionData > 10000000,
								 " &nbsp; ", as.character(round(ExpressionData,3)) ) 

		#    GeneId1 <- dget(paste(ArrayToolsPath, "/Fortran/GeneId1.rda", sep=""))
		load(paste(WorkPath, "/GeneId1.rda", sep=""))

		# Not sure if this is really necessary, but Amy copied it over when merging with original PrepareBinaryTreePredictionTableHTML.R file on 6/29/06.
		if (AnalysisType == "BinaryTreePrediction") Experiments <- Experiments[as.vector(unlist(ExpressionData.raw[1,-1])),]

		# Class Comparison; not-paired; not survival
		# ExpId - pheno
		if(!is.null(Experiments$pheno) & is.null(Experiments$id) & !exists(as.character(substitute(RedIDVariable)))) {
		   ExpressionTable.H1 <- paste("<TR><TH>&nbsp;<TH>Experiment<TH>",paste(Experiments$ExpId,collapse="<TH>")) # Qian 5/29/09
		   ExpressionTable.H2 <- paste("<TR><TD>&nbsp;<TD>Class<TD>",paste(Experiments$pheno,collapse="<TD>")) # Qian 5/29/09
		   TableTitle <- "Normalized Log-Transformed Gene Expressions for Significant Genes"
		   nHeaderLines  <- 2
		}

		# Class Comparison; paired; not survival
		# ExpId - pheno - id
		if(!is.null(Experiments$pheno) & !is.null(Experiments$id) & !exists(as.character(substitute(RedIDVariable))))  {
		   npairs <- length(Experiments$pheno)/2
		   ClassDiff <- paste(as.character(Experiments$pheno[1]),
					 as.character(Experiments$pheno[npairs+1]),sep="/")
		   Experiments.id <- Experiments$id[1:npairs]
		   ExpressionTable.H1 <- paste("<TR><TH>&nbsp;<TH>Pair<TH>",paste(Experiments.id,collapse="<TH>")) # Qian 5/29/09
		   TableTitle <- paste("Log-Ratio (",ClassDiff,") of Normalized Gene Expressions", " for Significant Genes",sep="") 
		   nHeaderLines  <- 1
		}

		# Class Comparison; not paired; survival
		# ExpId - phenoClass - phenoTime
		if(!is.null(Experiments$phenoClass) ) {
		   ExpressionOrder <- order(-as.double(as.character(Experiments$phenoClass)),Experiments$phenoTime)
		   Experiments <- experiments[ExpressionOrder,,drop = FALSE]
		   ExpressionData <- ExpressionData[,ExpressionOrder, drop = FALSE]
		   ExpressionTable.H1 <- paste("<TR><TH>&nbsp;<TH>Experiment<TH>",paste(Experiments$ExpId,collapse="<TH>")) # Qian 5/29/09
		   ExpressionTable.H2 <- paste("<TR><TD>&nbsp;<TD>Status<TD>",paste(Experiments$phenoClass,collapse="<TD>")) # Qian 5/29/09
		   ExpressionTable.H3 <- paste("<TR><TD>&nbsp;<TD>Time<TD>",paste(Experiments$phenoTime,collapse="<TD>")) # Qian 5/29/09
		   TableTitle <- "Normalized Log-Transformed Gene Expressions for Significant Genes"
		   nHeaderLines  <- 3
		}
		# Class Prediction; not paired
		# ExpId - ClassVariable
		if(!is.null(Experiments$ClassVariable) & is.null(Experiments$PairingVariable) ) {
		   ExpressionCode <- Experiments$ClassVariable != "predict"
		   Experiments <- Experiments[ExpressionCode ,]
		   #  LG, 09/05/03 ExpressionData <- ExpressionData[,ExpressionCode ]
		   ExpressionTable.H1 <- paste("<TR><TH>&nbsp;<TH>Experiment<TH>",paste(Experiments$ExpId,collapse="<TH>")) # Qian 5/29/09
		   ExpressionTable.H2 <- paste("<TR><TD>&nbsp;<TD>Class<TD>",paste(Experiments$ClassVariable,collapse="<TD>")) # Qian 5/29/09
		   TableTitle <- "Normalized Log-Transformed Median-Centered Gene Expressions for Significant Genes"
		   nHeaderLines  <- 2
		}
		# Class Prediction; paired
		# ExpId - ClassVariable - PairingVariable
		if(!is.null(Experiments$ClassVariable) & !is.null(Experiments$PairingVariable) )  {
		   ExpressionCode <- Experiments$ClassVariable != "predict"
		   Experiments <- Experiments[ExpressionCode ,]
		   #  LG, 09/05/03 ExpressionData <- ExpressionData[,ExpressionCode ]
		   npairs <- length(Experiments$ClassVariable)/2
		   ClassDiff <- paste(as.character(Experiments$ClassVariable[1]),
					 as.character(Experiments$ClassVariable[npairs+1]),sep="/")
		   Experiments.id <- Experiments$PairingVariable[1:npairs]
		   ExpressionTable.H1 <- paste("<TR><TH>&nbsp;<TH>Pair<TH>",paste(Experiments.id,collapse="<TH>")) # Qian 5/29/09
		   TableTitle <- paste("Log-Ratio (",ClassDiff,") of Normalized Gene Expressions", " for Significant Genes",sep="") 
		   nHeaderLines  <- 1
		}
		# Block Class Comparison; paired
		# ExpId - pheno - Block
		if( exists(as.character(substitute(RedIDVariable))) )  {
		   npairs <- length(Experiments$pheno)/2
		   ClassDiff <- paste(as.character(Experiments$pheno[1]),
						as.character(Experiments$pheno[npairs+1]),sep="/")

		   # Amy modified 7/4/06 to suppress warning message.
		   warnCurrent <- options()$warn
		   options(warn=-1)  # Suppress all warning messages, since the following 'if' condition produces 'NAs introduced by coercion' warning message if class label is character rather than numeric.
		   if(!is.na( as.double( as.character(Experiments$pheno[1]) ) ) ) {
			  ClassDiff <- paste(paste("class",as.character(Experiments$pheno[1])), paste("class",as.character(Experiments$pheno[npairs+1])),sep="/")
		   }
		   options(warn=warnCurrent)

		   if(!is.null(RefClass)) {
			  Experiments$ExpId <- ifelse(tolower(substring(as.character(Experiments$ExpId),1,9)) == "reference",
									 substring(as.character(Experiments$ExpId),1,9),Experiments$ExpId)
		   }

		   Experiments.id <- paste(Experiments$ExpId[1:npairs],Experiments$ExpId[(npairs+1):(2*npairs)],sep="/")

		   ExpressionTable.H1 <- paste("<TR><TH>&nbsp;<TH>Sample IDs (",ClassDiff,")<TH>",paste(Experiments.id,collapse="<TH>")) # Qian 5/29/09
		   TableTitle <- paste("Log-Ratio (",ClassDiff,") of Normalized Gene Expressions",
					  " for Significant Genes",sep="") 
		   nHeaderLines  <- 1
		}
		# Correlation
		# ExpId - Correlation
		if(!is.null(Experiments$Correlation) ) {

		   ExpressionOrder <- order(Experiments$Correlation)
		   Experiments <- experiments[ExpressionOrder,,drop = FALSE]
		   ExpressionData <- ExpressionData[,ExpressionOrder, drop = FALSE]

		   ExpressionTable.H1 <- paste("<TR><TH>&nbsp;<TH>Experiment<TH>",paste(Experiments$ExpId,collapse="<TH>")) # Qian 5/29/09
		   ExpressionTable.H2 <- paste("<TR><TD>&nbsp;<TH>Trait<TD>",paste(Experiments$Correlation,collapse="<TD>")) # Qian 5/29/09
		   TableTitle <- "Normalized Log-Transformed Gene Expressions for Significant Genes"
		   nHeaderLines  <- 2
		}

		ExpressionTable <- character(nHeaderLines+length(ExpressionData[,1]) )
		ExpressionTable[1] <- ExpressionTable.H1
		if(nHeaderLines > 1 ) ExpressionTable[2] <- ExpressionTable.H2
		if(nHeaderLines > 2 ) ExpressionTable[3] <- ExpressionTable.H3
		for( i in 1:length(ExpressionData[,1])) {
		   ExpressionTable[i+nHeaderLines] <- paste("<TR><TD>",i,"<TD>",  # Qian 5/29/09
			   GeneId1[ExpressionData.geneid[i] ],"<TD NOWRAP>",
			   paste(ExpressionData[i,] ,collapse="<TD NOWRAP>"))
		}
		cat( "<table border='1' class='sortable'>" # Qian 5/29/09 sortable
		  , paste("<P><BR><BR><U>Table.</U>",TableTitle,"</P>")
	#      , "<TABLE BORDER>"
		  , "<CAPTION ALIGN=left></CAPTION>"
		  , ExpressionTable
		  , "</TABLE>"
		  , file=DataTableFileName, sep="\n", append=TRUE
		  , "</body>"
		  , "</html>"
		)
	}
}
# End if-WriteExpressionData-loop 
