# GetComparisonResults ----------------------------------------------------

# From GetComparisonResults.R.
# Comments:
# 4/11/2017 Ting Remove affymetrix links for probeset IDs
# 3/10/2015 Ming-chung re-indent the code to make it more readable.
# 8/30/2012 Qian paired data show patient ID instead of array ID (such as Perou dataset).
# 8/29/2012 Qian Table 2 is not needed.
# 12/16/11 Ming-Chung fixed RVM option to show on HTML.
# 12/7/06 Yong modified to replace dget/dput with load/save for R2.4.
# 1/20/06 Ming-Chung modified to incorporate adjusted p-values.
#         fix the number of genes in expression table (ngenesExp). 
# 12/22/05 Deyun modified for adding a confidence text box and changing two check boxes to three radio buttons at forms
# 8/1/05 Ming-Chung
#       Change in the html output "Number of genes:" to 
#       "Number of genes used for random variance estimation:", and take out 
#       this line entirely when the random variance is not used, in order to 
#       avoid confusion with the following line "Number of genes that passed 
#       filtering criteria:".
# 8/19/03 Amy modified to remove output files from previous analysis.
GetComparisonResults <- function(geneIds, 
                                 ClassVariableName, 
                                 BlockVariableName,
                                 FilterAnalysisSet,
                                 IsSingleChannel,
                                 GeneListFileName,
                                 paired,
                                 ProportionOp="optUnivariateTest",
                                 SignifLFDR=0.05,
                                 GlobalTest=FALSE,
                                 DoGOOvE, 
                                 LogBase, 
                                 nspot,
                                 GenesTotal, 
                                 RunChromDist=FALSE,
                                 IngenuityRefClass='Automatically create the combination output',
                                 ArrayToolsPath,
                                 projectPath,
                                 outputName,
                                 AnalysisType, 
                                 WorkPath) {
  
  resList <- NULL
  UseFrames <- FALSE
  AnalysisName <- outputName

  OutputPath <- file.path(projectPath, "Output", AnalysisName)
  if (file.exists(OutputPath)) {
    unlink(paste(OutputPath, '/*', sep=''))
  } else {
    dir.create(OutputPath, recursive = TRUE)
  }
  
  #ExternalFileName <- paste(projectPath, "/Output/", AnalysisName, "/", AnalysisName, ".html", sep="")
  ChromDistFile <- paste(projectPath, "/Output/", AnalysisName, "/", AnalysisName, " -ChromDist.jpg", sep="")
  ExternalFileName <- paste(projectPath, "/Output/", AnalysisName, "/", AnalysisName, ".html", sep="")
  
  #Pass parameter, geneIds, to replace the file, GeneListFileName. 
  #Create genelist folder.
  #dir.create(paste(projectPath, "/Genelists", sep=""), showWarnings=FALSE)
  #dir.create(paste(projectPath, "/Genelists/AnalysisResults", sep=""), showWarnings=FALSE)
  
  unlink(ExternalFileName)
  unlink(ChromDistFile)
  
  #delete the existing genelist file with the same name. GeneListFileName defined in RunR.R. Yong 2/1/07
  #curgenelistfile<-paste(projectPath, "/Genelists/AnalysisResults/", GeneListFileName, sep="")
  #if (file.exists(curgenelistfile)) unlink(curgenelistfile)
  
  params <- scan(paste(WorkPath,"/class_perm_params.txt", sep=""), quiet=TRUE)
  DoExactPermTest <- params[1]
  NumPermutations <- params[2]
  NumPermutationsP<- params[3]
  RunPermP  <- NumPermutationsP  > 0
  RunGlobal <- NumPermutations > 0
  codeT <- params[4] == 1
  alphaT <- params[5]
  codeEFN <- params[6] == 1
  EFN <- params[7]
  codeEFG <- params[8] == 1
  alphaEF <- params[9]
  codePF <- params[10] == 1     
  alphaPF <- params[11]
  u <- params[12]
  g <- params[13] 
  codeReg <- params[14] 
  codeSurv <- params[15]
  ISEED <- params[16] 
  HasBlock <- params[17] == 1
  codeFoldChange <- params[18] == 1 
  FoldThresh <- params[19]
  AlphaPairwise <- params[20]
  RefClass <- NULL
  LR.line1 <- scan(paste(WorkPath,"/LR_data.txt", sep=""),nlines=1,quiet=TRUE)
  n <- scan(paste(WorkPath,"/LR_data.txt", sep=""),skip=1,nlines=1,quiet=TRUE)
  ngroups <- LR.line1[1]
  if(codeSurv == 1) nspot <- LR.line1[2]   #Amy modified on 4/11/03.
  paired <- LR.line1[3] == 1
  nsamp <- sum(n)
  GenesFiltered  <-  nspot
  nDeg <- 0
  GTest <- 1
  if(codeT) {
    Tres <- scan(paste(WorkPath,"/Tresult.txt", sep=""),sep=",",skip=1,nlines=1,quiet=TRUE)
    ngenesExp <- nDeg <- Tres[1]
    GTest <- Tres[2]
    PAllgenes <- scan(paste(WorkPath,"/pvalues.txt", sep=""), quiet=TRUE)
  }
  KSwarning <- FALSE
  # Modified Ming-Chung Li 6/8/05. Add verifying if RVM estimation successes
  regparfile <- paste(WorkPath,"/RegPar.txt", sep="")
  if(codeReg && file.exists(regparfile) && length(readLines(regparfile)) > 1) {
    RegPar <- scan(regparfile,sep=",",skip=1,nlines=1,quiet=TRUE)
    RegParA <- RegPar[1] 
    RegParB <- RegPar[2]
    RegParKS <- RegPar[3]
    KSwarning <- RegParKS > 0.05
  } else if (codeReg) {
    KSwarning <- TRUE
  }
  UseRVM <- 0 ; if (codeReg & !KSwarning){ UseRVM <- 1 } # added Jacob 03/05/08
  NexpG <-0
  gef <- 1
  GEFGTest <- 1
  if(codeEFG) {
    EFGres <- scan(paste(WorkPath,"/EFGresult.txt", sep=""), sep=",",skip=1,nlines=1,quiet=TRUE)
    NexpG<- EFGres[1]
    gef <- EFGres[2]
    GEFGTest <- EFGres[3]
  }
  NexpN <-0
  EFN <- 0
  GEFNGTest <- 1
  if(codeEFN) {
    EFNres <- scan(paste(WorkPath,"/EFNresult.txt", sep=""), sep=",",skip=1,nlines=1,quiet=TRUE)
    NexpN<- EFNres[1]
    EFN <- EFNres[2]
    GEFNTest <- EFNres[3]
  }
  k  <-0
  k1 <- 0
  k3 <- 0
  TooLong1 <- 0
  
  if(codePF & !GlobalTest) {
    PFres <- scan(paste(WorkPath,"/PFresult.txt", sep=""), sep=",",skip=1,nlines=1,quiet=TRUE)
    k1  <- PFres[1]
    k3  <- PFres[2]
    TooLong1 <- PFres[3]
  }
  
  ##Deyun added for form changes 12/22/05
  #Selected <-scan(paste(ArrayToolsPath,"/Fortran/param_file.rda", sep=""),what=list(x="", y=0, z=0), sep=",")
  pmt1 <-ProportionOp
  
  ##k <- max(k1,k3)
  ##Deyun modified for form changes 12/22/05
  if (pmt1=="optProportion"){  
    k <- k3
  }
  if (pmt1=="optNumber"){  
    k <- k1
  }
  if(codePF & !GlobalTest) ngenesExp <- k
  
  
  if(paired) {TestType <- "Paired T-test"
  } else { 
    if (ngroups == 2) { TestType <- "Two-sample T-test"
    } else { TestType <- "F-test"} }
  
  if(HasBlock) {TestType <- "Randomized block design"}
  
  if(codeSurv == 1) TestType <- "Cox proportional hazards model, Wald Statistic"
  
  nGenesInTable <- max(nDeg,NexpN,NexpG,k)
  
  
  #experiments <- dget(paste(ArrayToolsPath, "/Fortran/experiments.rda", sep=""))
  load(paste(WorkPath, "/experiments.rda", sep="")) #use load/save method, Yong 12/6/06. Use WorkPath 12/15/06
  
  if(codeSurv == 0) {  
    classlabels <- unique(sort(experiments$pheno))
    classsize <- as.integer(table(experiments$pheno))
  } else {
    classlabels <- unique(sort(experiments$phenoClass))
    source(paste(ArrayToolsPath, "R/Survivalfuns.R", sep="/"))   
    KMcurve.fun(WorkPath, paste(projectPath, "/Output/", AnalysisName, "/kmcurve", sep=""))
  }
  
  # MLI, add pairwise comparison option. 12/24/08
  # 1.Check if it is one way anova and has at least 3 levels
  # 2.Run pairwise comparison and save result to pairwiseCC.rda
  # 3.Add code in PrepareGeneTable.R (line465) to display the result in gene table
  # 4.Add code on line 484 to display the form show/hide button.
  # 5.Add code to sort the class labels by the mean.
  if (codeSurv==0 & !HasBlock & ngroups >= 3 & nGenesInTable > 0) {
    if (!exists("AlphaPairwise")) AlphaPairwise <- .01 # set to default value
    #y <- experiments$pheno # if we want to print the class label
    y <- as.integer(experiments$pheno) 
    ExpressionData <- read.table(paste(WorkPath, "/GeneExpData.txt", sep=""))[, -1]
    ExpressionData <- as.matrix(ExpressionData)
    ExpressionData[ExpressionData < -100000 | ExpressionData > 10000000] <- NA
    pairwiseCC <- vector("list", nGenesInTable)
    for(i in 1:nGenesInTable) {
      tmp <- pairwise.t.test(ExpressionData[i,], y, p.adj = "none", pool.sd = TRUE)
      if (any(tmp$p.value < AlphaPairwise, na.rm=TRUE)) {    
        ind <- which(tmp$p.value < AlphaPairwise, arr.ind=TRUE)
        pairlabel <- cbind(rownames(tmp$p.value)[ind[,1]], colnames(tmp$p.value)[ind[,2]])
        cmean <- tapply(ExpressionData[i,], y, mean, na.rm=TRUE)
        paircmean <- t(apply(pairlabel, 1, function(x) {
          cmean[match(x, names(cmean))]
        }))       
        paircmean.order <- t(apply(paircmean, 1, order))
        pairlabel.sort <- matrix(nr=nrow(pairlabel), nc=2)
        for(j in 1:nrow(pairlabel)) pairlabel.sort[j,] <- pairlabel[j, paircmean.order[j,]]
        pairwiseCC[[i]] <- apply(pairlabel.sort, 1, function(x) paste("(", paste(x, collapse=", "), ")", sep=""))
      }
    }    
    names(pairwiseCC) <- seq(pairwiseCC)
    unlink(paste(WorkPath, "/pairwiseCC.rda", sep=""))
    save(pairwiseCC, file=paste(WorkPath, "/pairwiseCC.rda", sep=""))
  }
  
  dynamicplotfilename <- ifelse(ngroups ==2 & HasBlock !=1, "VolcanoPlot.html", "PCPlot.html")
  
  # Local FDR
  # Require CRAN's 'quadprog' package. Need to update CRANPackages.txt
  RunLFDR <- FALSE
  if (ProportionOp == "optLocalFDR") {  
    RunLFDR <- TRUE
    PKGNAME <- "quadprog"
    IsBioC <- FALSE
    if (!any(installed.packages()[,1] == PKGNAME)) {
      #source(paste(ArrayToolsPath,"/R/BiocUtil.R", sep=""), echo = FALSE)
      if (!LoadRPackage(PKGNAME, isBioconductor=IsBioC)) {
        require(tcltk)
        tkmessageBox(message=paste(PKGNAME, "cannot be downloaded or installed."),
                     icon="info",type="ok")
        RunLFDR <- FALSE
      }
    }
    if (RunLFDR) {
      #source(paste(ArrayToolsPath, "R/SWeger-RCode.R", sep="/"))   
      GlobalTest <- FALSE
      PAllgenes <- scan(paste(WorkPath,"/pvalues.txt", sep=""), quiet=TRUE)
      # Up P-values if it is exactly 0; otherwise it causes a problem in nlpden
      PAllgenes <- pmax(PAllgenes, 1.0e-8)
      # Convert t to z
      # Since fortran does not output T-stat, we need to do calculation to get the sign.
      if (!paired) {
        load(paste(WorkPath,"/meanLR1.rda", sep=""))
        meanLR1 <- meanLR
        load(paste(WorkPath,"/meanLR2.rda", sep=""))
        meanLR2 <- meanLR
        z <- qnorm(1 - PAllgenes/2) * sign(meanLR1 - meanLR2)
      } else {
        load(paste(WorkPath,"/meandiff.rda", sep=""))
        z <- qnorm(1 - PAllgenes/2) * sign(meandiff)
      }	   
      # Compute f(z)
      fhat <- nlpden(z[is.finite(z)]) # handle NAs from meanLR
      fz <- sapply(z, function(x) approx(fhat$x_axis, fhat$density_estimate, xout = x)$y)
      f0z <- dnorm(z)
      # local fdr
      lfdr <- f0z / fz
      ind.deg.lfdr <- which(lfdr <= SignifLFDR)
      ind.deg.lfdr <- ind.deg.lfdr[order(PAllgenes[ind.deg.lfdr])] # Sort by p-values
      nDeg <- length(ind.deg.lfdr)
      nGenesInTable <- nDeg # used in creating DEGs.txt. See line ~ 500.
      ngenesExp <- nDeg     # used in Input_to_output_function.R to create Expression data table in html
      #cat("Number of DEGs by using an empirical Bayes method: ", length(ind.deg.lfdr), " (cutoff=.05)\n")	   
      classifiers <- data.frame(Index = ind.deg.lfdr, pvalue = PAllgenes[ind.deg.lfdr], lfdr = lfdr[ind.deg.lfdr])
      
      png(file.path(projectPath, "Output", AnalysisName, "DensityPlot.png"))
      ylim <- max(c(f0z, fz), na.rm=TRUE)
      hist(z, freq=FALSE, ylim = c(0, ylim))
      ord <- order(z)
      lines(z[ord], f0z[ord], col="red")
      lines(z[ord], fz[ord], col="blue")
      legend("topright",  c(expression(f[0](z)), "f(z)"), col=c("red", "blue"), lwd=2)
      dev.off()
    }
  }
  
  ## WRITE OUTPUT:
  cat("<!-- This is an HTML document.  Please make sure the filename has either"
      , "     an .htm or .html extension, and view this file in a web browser. -->"
      , ""
      , "<HTML>"
      , "<HEAD>"
      , ifelse(codeSurv == 0, "<TITLE>Class Comparison Results</TITLE>", "<TITLE>Survival Analysis Results</TITLE>")
      , "<STYLE>"
      , "P {font-size: 9pt}"
      , "TABLE {font-size: 9pt}"
      , "</STYLE>"
      , paste("<script src='file:///", ArrayToolsPath, "/Fortran/sorttable.js'></script>", sep="") #Qian 5/29/09 sortable
      , "</HEAD>"
      , "<BODY>"
      , "<SCRIPT>"
      , "function popUp(text) {"
      , "   var prop ="
      , "   \"location=no,scrollbars=yes,menubars=no,toolbars=no,resizable=yes\";"
      , "   popup = window.open(\"\",\"mywin\",prop);"
      , "   popup.document.open();"
      , "   popup.document.write(text);"
      , "   popup.document.close();"
      , "   popup.focus();"
      , "   }"
      , "   </SCRIPT>"
      , ifelse(codeSurv == 0, paste("<A HREF= ",'"file:///',paste(ArrayToolsPath, "/Fortran/ClassComparisonHelp.html", sep=""),'"'
                                    ," TARGET=_blank>HELP </A>",sep=""),
               paste("<A HREF= ",'"file:///',paste(ArrayToolsPath, "/Doc/HTML/SurvivalAnalysisHelp.html", sep=""),'"'
                     ," TARGET=_blank>HELP </A>",sep=""))
      #Qian 5/4/09
      #, ifelse(codeSurv == 0, "<h2 align='center'>Class Comparison using BRB-ArrayTools</h2>", "<h2 align='center'>Survival Analysis Results</h2>")
      , "<h3>Contents</h3>"
      , "<ul>"
      , "<li><a href='#Description'><b>Description of the problem</b></a></li>"
      , ifelse(codeSurv == 0,"<li><a href='#Genes'><b>Genes which are differentially expressed among classes</b></a></li>",
               "<li><a href='#Genes'><b>Genes significantly associated with survival</b></a></li>")
      , ifelse(DoGOOvE, "<li><a href='#OEtable'><b>View 'Observed v. Expected' table</b></a></li>", "")
      , ifelse(RunChromDist, "<li><a href='#ChromDist'><b>View Chromosomal Distribution Graph</b></a></li>", "")
      , ifelse(RunLFDR, "<li><a href='#Density'><b>Density plot of Z-statistic for local false discovery rate</b></a></li>", "")		
      , "<li><a href='ExpressionTable.html'><b>View expression of significant genes</b></a></li>"
      , ifelse(codeSurv == 0,"<li><a href='heatmap.html'><b>View clustered heatmap of significant genes</b></a></li>", "")
      , ifelse(codeSurv == 0,paste("<li><a href=\'", dynamicplotfilename, "\'><b>View dynamic ",   
                                   ifelse(ngroups ==2 & HasBlock !=1, "volcano", "parallel coordinate"), 
                                   " plot of significant genes</b></a></li>", sep = ""), "")
      #, "<li><a href='#Filtering'><b>Filtering parameters</b></a></li>"
      , "</ul>"
      , "<hr size='5' width='100%' noshade />"
      , "<h3><a name='Description'><b><u>Description of the problem:</u></b></a></h3>"
      #EndQian                                
      , "<P>"
      , ifelse(codeSurv == 0,paste("Number of classes:",ngroups),"")
      , "<BR>"
      , if (codeReg & !KSwarning) paste("Number of genes used for random variance estimation:",GenesTotal, "<BR>")
      , paste("Number of genes that passed filtering criteria:",GenesFiltered )
      , "<BR>"
      , paste("Type of univariate test used:",TestType,ifelse(UseRVM == 0,"", "(with random variance model)"))  # changed "codeReg == 0" into "UseRVM == 0" Jacob 03/05/08
      , "<BR>"
      ," ", file=ExternalFileName, sep="\n", append=FALSE)
  
  #Qian 5/8/09
  if (codeSurv == 0)  {
    if(exists(as.character(substitute(ClassVariableName))) ) {
      cat(
        paste("Class variable :",ClassVariableName)
        , "<BR>"
        ," ", file=ExternalFileName, sep="\n", append=TRUE) 
    }
    if(HasBlock & exists(as.character(substitute(BlockVariableName))) ) {
      cat(
        paste("Block variable :",BlockVariableName)
        , "<BR>"
        ," ", file=ExternalFileName, sep="\n", append=TRUE) 
    }
    if(exists(as.character(substitute(RedIDVariable))) ) {
      RefCode <- substring(tolower(as.character(experiments$ExpId)),1,9) == 
        "reference"
      if(length(RefCode[RefCode]) > 0) {
        RefClass <- experiments$pheno[RefCode]
        RefClass <- RefClass[1]
      } else {RefClass <- NULL}
      cat(
        paste("Red-labeled Sample ID :",RedIDVariable)
        , "<BR>"
        ,paste("Green-labeled Sample ID :",GreenIDVariable)
        , "<BR>"
        ,paste("Red-labeled class :",RedClassVariable)
        , "<BR>"
        ,paste("Green-labeled class :",GreenClassVariable)
        , "<BR>"
        ," ", file=ExternalFileName, sep="\n", append=TRUE) 
    }
  } else {
    cat(
      paste("Survival Time variable :",TimeVariable)
      , "<BR>"
      ,paste("Survival Status variable :",StatusVariable)
      , "<BR>"
      ," ", file=ExternalFileName, sep="\n", append=TRUE) 
  }
  #EndQian
  
  # Ming-chung add a statement if RVM was not selected. 12/16/2011
  if (!codeReg) 
    cat("Random variance model for univariate tests: OFF<BR>", file=ExternalFileName, sep="\n", append=TRUE) 
  if (codeReg & !KSwarning)  
    cat(paste("Random variance model parameters: a=",
              RegParA,",  b=",RegParB, ", Kolmogorov-Smirnov statistic=", RegParKS)
        , "<BR>"
        ," ", file=ExternalFileName, sep="\n", append=TRUE) 
  
  if (KSwarning)  
    cat(paste("WARNING: Distribution assumptions underlying the random variance model are not satisfied. Random variance model was not used. ", if (exists("RegParKS")) paste("Kolmogorov-Smirnov statistic=", RegParKS), sep=" ")
        , "<BR>"
        ," ", file=ExternalFileName, sep="\n", append=TRUE) 
  
  if (codePF & !GlobalTest & !DoExactPermTest)  
    cat(paste("Multivariate Permutations test was computed based on",
              NumPermutations,"random permutations")
        , "<BR>"
        ," ", file=ExternalFileName, sep="\n", append=TRUE)
  
  if (codePF & !GlobalTest & DoExactPermTest)  
    cat(paste("Exact Multivariate Permutations test was computed based on",
              NumPermutations,"available permutations")
        , "<BR>"
        ," ", file=ExternalFileName, sep="\n", append=TRUE) 
  
  if (RunPermP & !DoExactPermTest) 
    cat(paste("Permutation p-values for significant genes were computed based on",  
              NumPermutationsP,"random permutations")
        , "<BR>" 
        ," ", file=ExternalFileName, sep="\n", append=TRUE)
  
  if (RunPermP & DoExactPermTest) 
    cat(paste("Exact permutation p-values for significant genes were computed based on",  
              NumPermutationsP,"available permutations")
        , "<BR>" 
        ," ", file=ExternalFileName, sep="\n", append=TRUE)
  if (codeT && pmt1 != "optLocalFDR") cat(
    paste("Nominal significance level of each univariate test:",alphaT,"<BR>") #Qian 5/4/09
    ," ", file=ExternalFileName, sep="\n", append=TRUE)
  if (pmt1 == "optLocalFDR") cat(
    paste("Maximum allowed local false discovery rate:", SignifLFDR,"<BR>") #MLI 2/18/15
    ," ", file=ExternalFileName, sep="\n", append=TRUE)
  if (codeEFG) cat(
    "<BR>", paste("Maximum allowed expected proportion of false-positive genes:",alphaEF)
    ," ", file=ExternalFileName, sep="\n", append=TRUE)
  
  if (codeEFN) cat(
    "<BR>", paste("Maximum allowed expected number of false-positive genes:",EFN)
    ," ", file=ExternalFileName, sep="\n", append=TRUE)
  
  ##Deyun modified for form changes 12/22/05; Ming-Chung 1/20/06    
  if (codePF & !GlobalTest & pmt1=="optProportion"){  
    TextToPrint <- paste("Maximum allowed proportion of false-positive genes:",g)
  } else if (codePF & !GlobalTest & pmt1 == "optNumber") {
    TextToPrint <- paste("Maximum allowed number of false-positive genes:",u)
  }
  if (codePF & !GlobalTest & pmt1 != "optLocalFDR") cat(
    "<BR>", paste("Confidence level of false discovery rate assessment:",100*round(1-alphaPF,3),"%"),
    "<BR>", TextToPrint," ", file=ExternalFileName, sep="\n", append=TRUE)
  #Qian 5/11/09
  cat("</p>"
      , "<HR SIZE=5 WIDTH=\"100%\" NOSHADE>"
      , ifelse(codeSurv == 0,"<h3><a name='Genes'><b><u>Genes which are differentially expressed among classes:</u></b></a></h3>", #Qian 4/9/09
               "<h3><a name='Genes'><b><u>Genes significantly associated with survival:</u></b></a></h3>") #Qian 4/9/09
      , "<p>"
      , file=ExternalFileName, sep="\n", append=TRUE)
  
  if (codeT) {
    cat(paste("Number of genes significant at",alphaT,
              "level of the univariate test:",nDeg,"<br />")
        ," ", file=ExternalFileName, sep="\n", append=TRUE)
  } else if (codePF & !GlobalTest & pmt1=="optProportion") {
    cat(paste("Number of significant genes by controlling the proportion of false positive genes:", k3,"<br />"),
        " ", file=ExternalFileName, sep="\n", append=TRUE)
  } else if (pmt1 == "optLocalFDR" ) {
    cat(paste("Number of genes significant at", SignifLFDR,
              "level of the local false discovery rate:",nDeg,"<br />")
        ," ", file=ExternalFileName, sep="\n", append=TRUE)		   
  } else {
    cat(paste("Number of significant genes by controlling the number of false positive genes:", k1,"<br />"),
        " ", file=ExternalFileName, sep="\n", append=TRUE)
  }           
  #EndQian
  
  # by Rich's order 2/28/06 since the number of permutations is not in gui
  # put it back 7/20/06 with 1000 (hard coded in class_perm_params.txt) permutations.
  if (GlobalTest) {
    classorsurvival <- ifelse(codeSurv ==1,
                              "there is no relationship between expression profile and survival:",
                              "there are no real differences between the classes:")          
    cat(paste("Global test: probability of getting at least",
              nDeg,"genes significant by chance (at the", alphaT,"level) if",
              classorsurvival, GTest,"<br />")
        ," ", file=ExternalFileName, sep="\n", append=TRUE)
  }
  
  if (codeEFN) cat(
    paste("Number of genes with the smallest p-values that are expected to",
          "contain no more than",EFN, "false-positive genes:",NexpN,"<br />")
    ," ", file=ExternalFileName, sep="\n", append=TRUE)
  
  if (codeEFN & RunGlobal & NexpN > 0) cat(
    paste("Estmated probability of having at least", NexpN,
          "genes with less than", EFN,"false-positive genes:",GEFNTest,"<br />")
    ," ", file=ExternalFileName, sep="\n", append=TRUE)
  
  if (codeEFG) cat(
    paste("Number of genes with the smallest p-values that are expected to",
          "contain no more than",100*alphaEF, "% false-positive genes:",NexpG,"<br />")
    ," ", file=ExternalFileName, sep="\n", append=TRUE)
  
  if (codeEFG & RunGlobal & NexpG > 0) cat(
    paste("Estmated probability of having at least", NexpG,
          "genes with less than", 100*alphaEF,"% false-positive genes:",GEFGTest,"<br />")
    ," ", file=ExternalFileName, sep="\n", append=TRUE)
  
  
  if (nGenesInTable == 0) {
    cat(paste("No genes were significant by any of the chosen criteria."),
        file=ExternalFileName)
  } else {
    ##Deyun modified for form changes 12/22/05    
    fileNamex = paste(WorkPath,"/DEGs.txt", sep="")
    xy <- readLines(fileNamex, n=-1)
    unlink (fileNamex)
    cat (xy[1],xy[2], file=fileNamex, sep="\n")
    Nx <- 2 + nGenesInTable
    for(i in 3:Nx) cat (xy[i],file=fileNamex, sep="\n", append=TRUE )
    if (!RunLFDR) {  
      classifiers <- read.table(paste(WorkPath,"/DEGs.txt", sep=""), header=FALSE, skip=2)
      if(codeSurv == 0) {
        names(classifiers) <- c("Index","test","pvalue","permpvalue","EFpvalue","EFN") 
      } else {
        names(classifiers) <- c("Index","test","pvalue","permpvalue","EFpvalue","EFN","SD","b") 
      }
    } 
    
    if (nrow(classifiers) != nGenesInTable) {
      cat("R ERROR: Error reading classifiers from Fortran output.", file=ExternalFileName)
    } else {
      ## WRITE OUTPUT:
      classifiers$PFD <- rep("NO",length(classifiers[,1]))
      classifiers$PFD[1:k] <- "Yes"
      GetGenesHTMLOutput <- GetGenesHTML(index=classifiers$Index,
                                         classlabels=classlabels,
                                         paired=paired,
                                         RefClass = RefClass,
                                         univtest = codeT,
                                         WorkPath=WorkPath,
                                         classifiers=classifiers,
                                         LogBase=LogBase,
                                         projectPath=projectPath,
                                         PAllgenes=PAllgenes,
                                         RunLFDR=RunLFDR,
                                         RunPermP=RunPermP,
                                         IsSingleChannel=IsSingleChannel,
                                         AnalysisName=AnalysisName,
                                         AnalysisType=AnalysisType,
                                         DoGOOvE=DoGOOvE)
      
      classifierTable <- GetGenesHTMLOutput$GeneIdForIngenuity
      names(classifierTable) <- gsub("<NOBR>|</NOBR>|<BR>","",names(classifierTable))
      # replace class i with correpsonding class labels for better understanding
      for (i in 1:length(classlabels)){
        names(classifierTable) <- gsub(paste("class  ", i, sep = ''), classlabels[i], names(classifierTable))
      }
      rownames(classifierTable) <- as.character(1:nrow(classifierTable))
      resList <- c(resList, list(classifierTable=classifierTable))
      
      cat("<U>Table 1</U> - Sorted by p-value of the univariate test. <BR>" #Qian 5/11/09
          , ifelse(codeSurv==0 && paired==FALSE, outputClassname(classlabels, classsize), ""), "<BR>" # Yong added 12/15/06
          , file=ExternalFileName, sep="\n", append=TRUE)
      
      if (codeT) 
        cat(paste("The first",nDeg,"genes are significant at the nominal",alphaT,
                  "level of the univariate test",
                  if (codeFoldChange) paste("with the fold change", FoldThresh))
            , file=ExternalFileName, sep="\n", append=TRUE)
      
      if (codeEFN) 
        cat(paste("The first",NexpN, "genes are expected to",
                  "contain no more than",EFN, "false discoveries")
            ,file=ExternalFileName, sep="\n", append=TRUE)
      
      
      if (codeEFG) 
        cat(paste("The first",NexpG, "genes are expected to",
                  "contain no more than",100*alphaEF, "% false discoveries")
            , file=ExternalFileName, sep="\n", append=TRUE)
      
      
      if (codePF & !GlobalTest) {
        if (pmt1 == "optNumber"){   ##Deyun modified for form changes 12/22/05  
          message.k1 <- paste("With probability of",100*round(1-alphaPF,3),"% the first", k1, "genes contain no more than",u, "false discoveries.")
          
          if(alphaPF == 0.5) message.k1 <- paste("The predicted number of false discoveries among the first",
                                                 k1, "genes is",u)
          if(u >= nspot)  message.k1 <- paste("The entire gene list contains only",nspot,"genes.",
                                              "Therefore, the entire gene list contains less than",u,"false discoveries.")
          
          cat("<BR>",message.k1," ",file=ExternalFileName, sep="\n", append=TRUE)
        }
        if (pmt1=="optProportion"){ ##Deyun modified for form changes 12/22/05    
          if(alphaPF == 0.5) {
            cat( "<BR>", paste("The predicted proportion of false discoveries among the first ",
                               k3, " genes is ",100*round(g,2),"%.",sep=""),
                 ifelse(TooLong1 == 0 || k3 == nspot," ",paste("Further extension",
                                                               "of the list was halted because the list would contain more",
                                                               "than 100 false discoveries")), 
                 file=ExternalFileName, sep="\n", append=TRUE)
          } else {
            cat( "<BR>", paste("With probability of ",100*round(1-alphaPF,3),"% the first ",
                               k3, " genes contain no more than ",100*round(g,2),"% of false discoveries. ",sep=""),
                 ifelse(TooLong1 == 0 || k3 == nspot," ",paste("Further extension",
                                                               "of the list was halted because the list would contain more",
                                                               "than 100 false discoveries")), 
                 file=ExternalFileName, sep="\n", append=TRUE)
          }
        } ## if (pmt1=="optProportion"){  
      }
      if (ngroups >= 3 && nGenesInTable > 0) {
        cat("<BR>The 'Pairwise significant' column shows pairs of classes with significantly different 
            gene expression at alpha = ", AlphaPairwise, ". Class labels in a pair are ordered (ascending) by their averaged gene expression.\n", file=ExternalFileName, append=TRUE)  
      }  
      if (codeSurv == 1) 
        cat("<BR>", paste("Hazard ratio is the ratio of hazards for a two-fold change ",
                          "in the gene expression level. It is equal to exp(b) where",
                          " b is the Cox regression coefficient. SD is the standard",
                          " deviation of the log 2 of the gene expression level.")
            ," ", file=ExternalFileName, sep="\n", append=TRUE)
      
      cat("</P>",  file=ExternalFileName, sep="\n", append=TRUE)
      
      ####### BEGIN SECTION TO WRITE NETAFFX BATCH QUERY LINKS
      CreateNetAffxBatchQuery <- FALSE
      if (CreateNetAffxBatchQuery) {
        ProbeSets <- GetGenesHTMLOutput$ProbeSets
        if ( sum(!is.na(ProbeSets)) > 0 ) {
          #INSERT LINKS FOR BATCH QUERY OF NETAFFX.
          NumProbeSets <- length(ProbeSets)
          ChunkSize <- 50
          NumChunks <- ceiling(NumProbeSets/ChunkSize)
          
          cat("<P>",  file=ExternalFileName, sep="\n", append=TRUE)
          if (NumChunks==1) {
            CreateNetAffxBatchLinks(1, NumProbeSets, TRUE)
          } else {
            for (i in 1:(NumChunks-1)) CreateNetAffxBatchLinks((i-1)*ChunkSize+1, i*ChunkSize, FALSE)
            CreateNetAffxBatchLinks((NumChunks-1)*ChunkSize+1, NumProbeSets, TRUE)
          }
          cat("</P>",  file=ExternalFileName, sep="\n", append=TRUE)
        }
      }
      ####### END SECTION TO WRITE NETAFFX BATCH QUERY LINKS
      #Qian 8/29/2012 Table 2 is not needed.
      #    if (paired) {
      #      cat(GetGenesHTMLOutput$GeneId1
      #        , paste("<P><BR><BR><U>Table 2</U> - Sorted by mean difference:</P>")
      #        , GetGenesHTMLOutput$GeneId2
      #        , file=ExternalFileName, sep="\n", append=TRUE
      #      )
      #    } else {
      cat(GetGenesHTMLOutput$GeneId1, file=ExternalFileName, sep="\n", append=TRUE)
      #    }
      #EndQian
      
      #Qihao added codes to output a file for Ingenuity Pathway Analysis. 7/15/2013
      #Qihao fixed #515.
      if((codeSurv == 0)&&(!(paired))){
        IngenuityOutput<-paste(projectPath, "/Output/", AnalysisName,"/BRBArrayToolsOutputForIPA.txt", sep="")
        cat("<HR SIZE=5 WIDTH=\"100%\" NOSHADE>"
            ," "
            #,"Export results to Ingenuity IPA:"
            ,"The genes in the above table were saved as a list of genes along with their p-values and fold changes into a text file, BRBArrayToolsOutputForIPA.txt. This text file could be directly loaded into the Ingenuity IPA tool, and you could find the file "
            ,"<a href='",paste("file:///",projectPath, "/Output/", AnalysisName,sep=''),"'>"
            ,'here',"</a>."
            ,file=ExternalFileName, sep="\n", append=TRUE)
        #save.image('c:/temp/test.rdata')
        #Qihao modified codes to fix #599 7/20/2015
        output.ingenuity<-try(CreateIPAOutput(IngenuityOutput,GetGenesHTMLOutput$GeneIdForIngenuity,IngenuityRefClass,UseCorrelationAsFoldChange=FALSE,classlabels=classlabels))
        if (is(output.ingenuity,"try-error")==TRUE) { cat("Error: Ingenuity output file could not be created.",geterrmessage(),file=ExternalFileName,sep="\n",append=TRUE) }
      }
      
      if (DoGOOvE){   #Amy modified 'if' condition 10/8/03.
        #    if (length(GetGenesHTMLOutput$OEhtml) > 1) {  #If GOanalysis was done.
        #Qian 2/23/09
        if(!is.na(grep("minimum criteria", GetGenesHTMLOutput$OEhtml) ||
                  grep("not be computed", GetGenesHTMLOutput$OEhtml))) {
          cat("<P></P>"
              , "<HR SIZE=5 WIDTH=\"100%\" NOSHADE>"
              , "<P></P>"
              , "<a name='OEtable'>"
              , GetGenesHTMLOutput$OEhtml
              , "</a>"
              , file=ExternalFileName,sep="\n",append=TRUE)
        } else {
          cat( "<P></P>"
               , "<HR SIZE=5 WIDTH=\"100%\" NOSHADE>"
               , "<P></P><B><U>"
               , paste("<a name='OEtable'>'Observed v. Expected' table of GO classes and parent classes,",
                       "in list of", length(classifiers$Index), "genes shown above:</a>")
               , "</U></B><P></P>"
               , GetGenesHTMLOutput$OEhtml
               , file=ExternalFileName, sep="\n", append=TRUE)
        }
        #EndQian
      }
      if (RunChromDist && file.access(ChromDistFile, mode=4)==0) {  #If file is read-accessible.
        cat( "<P></P>"
             , "<HR SIZE=5 WIDTH=\"100%\" NOSHADE>"
             , "<P></P>"
             , paste("<a name='ChromDist'><IMG SRC=\"", AnalysisName, " -ChromDist.jpg", "\"></a>", sep="")
             , file=ExternalFileName, sep="\n", append=TRUE)
      }
      if (RunLFDR) {
        cat("<hr size='5' width='100%' noshade />", 
            "<h3><a name='Density'><b><u>Density plot of Z-statistic for local false discovery rate:</u></b></a></h3>",			
            file=ExternalFileName, sep="\n", append=TRUE)	
        cat("<p><img src = 'DensityPlot.png'><BR>",
            "f<sub>0</sub>(z): the density of the standard normal distribution. <br> ",
            "f(z): the estimated density of the Z-statistic used in computing the local false discovery rate.</p>",		
            file=ExternalFileName, sep="\n", append=TRUE)					
      }		
      cat("<hr size='5' width='100%' noshade />", 
          "<H3><a href='ExpressionTable.html'><b>View expression of significant genes</b></a></H3>",
          file=ExternalFileName, sep="\n", append=TRUE)
      if (codeSurv == 0) {
        cat("<H3><a href='heatmap.html'><b>View clustered heatmap of significant genes</b></a></H3>",
            "<H3>", paste("<a href=\'", dynamicplotfilename, "\'><b>View dynamic ",   
                          ifelse(ngroups ==2 & HasBlock !=1, "volcano", "parallel coordinate"), 
                          " plot of significant genes</b></a></H3>", sep = ""),
            file=ExternalFileName, sep="\n", append=TRUE)
      }
      } # end of if (nrow(classifiers) != nGenesInTable)
  } # end of if (nGenesInTable == 0)
  
  if (UseFrames) { #CREATE FRAME FILE, WHICH WILL BE OPENED UP IN INTERNET EXPLORER BY THE CALLING DOS SCRIPT.
    cat("<HTML>"
        , "<FRAMESET ROWS=\"99.9%,.1%\">"
        #    , "  <FRAMESET COLS=\"75%,25%\">"
        , "  <FRAMESET COLS=\"99.1%,.1%\">"
        , paste("    <FRAME NAME=\"Main\" SRC=\"", AnalysisName, ".html\">", sep="")
        , "    <FRAME NAME=\"Annotation\">"
        , "  </FRAMESET>"
        , "  <FRAME NAME=\"DummyFrame\">"
        , "</FRAMESET>"
        , "</HTML>"
        , file=FrameName, sep="\n", append=FALSE
    )
  } else {  #OPEN UP THE MAIN HTML FILE USING THE DEFAULT BROWSER.
    #  if (regexpr("NT", as.list(Sys.info())$release)>0) {
    #    shell(cmd=paste("\"", ExternalFileName, "\"", sep=""),
    #      translate=TRUE, wait=FALSE)
    #  } else {
    #    shell(cmd=paste("start \"", ExternalFileName, "\"", sep=""),
    #      translate=TRUE, wait=FALSE)
    #  }
  }
  
  if (codeSurv==0) {
    ##***BEGIN HEATMAP CODE: Tilman 8/19/08
    #
    #try.func <- try(source(paste(ArrayToolsPath, "R/CCheatmap.R", sep="/")),silent=TRUE) #--> Read-in function code and relevant data
    if (!RunLFDR) {
      try.data <- try(expdata <- read.table(paste(WorkPath, "GeneExpData.txt", sep="/"), header=FALSE)[, -1],silent=TRUE)
    } else {
      # See also <Input_to_output_function.R> lines 825-827
      ExpressionData <- as.matrix(read.delim(file.path(WorkPath, "LR_data.txt"), skip=2, header=FALSE))
      if (codeReg == 1) ExpressionData <- ExpressionData[ExpressionData[, ncol(ExpressionData)]==1, -ncol(ExpressionData)]
      ExpressionData <- ExpressionData[ind.deg.lfdr, , drop = FALSE]
      try.data <- expdata <- ExpressionData
      if (nDeg == 0)   class(try.data) <- "try-error"
    }  
    # GeneExpData nsiggenes x nfilterarrays
    #try.labs <- NULL #Qian 12/11/09
    #Pass parameter, geneIds, to replace the file, GeneListFileName. 
    curgenelistfile<-GeneListFileName
    if (nGenesInTable > 0) {
      genelabs = geneIds[FilterAnalysisSet==1,,drop=FALSE]
      genelabs = genelabs[classifiers$Index,,drop=FALSE]
    }
      #try.labs <- try(genelabs <- read.table(curgenelistfile,stringsAsFactors=FALSE,header=TRUE,sep="\t",na.strings=c("NA",""),fill=TRUE),silent=TRUE)
    # curgenelisfile is used to get the gene symbols for sig genes. Can we trust it?
    #try.cls <- try(genecls <- read.table(paste(WorkPath, "ExpDescWkSht.txt", sep="/"),stringsAsFactors=FALSE,header=TRUE,sep="\t",na.strings=c("NA",""),fill=TRUE),silent=TRUE) #Qian 4/29/09 doesn't need.
    # ExpDescWkSht nallarrays rows
    
    #--> Error handling code. If there is a problem with importing any files for the heatmap or there are too few significant
    #    genes, the results from the class comparison analysis will be displayed in the HTML file as normal, but the heatmap
    #    will not be created. Heatmap errors are reported in the class comparison HTML results file, in place of the link.
    #    The retrieval and printing of any heatmap error messages is done in Input_to_output_function.R.
    heatmap.error.text <- c(paste("Function file '",ArrayToolsPath,
                                  "R/CCheatmap.R' not found, or there was an error loading the file",sep=""),
                            paste("Data file '",WorkPath,"/GeneExpData.txt' not found, or there was an error loading the file",sep=""),
                            paste("Genelist file '",curgenelistfile,"' not found, or there was an error loading the file",sep=""),
                            paste("Experiment descriptors file '",WorkPath,"/ExpDescWkSht.txt' not found, or there was an error loading the file",sep=""),
                            "Heatmap will only be created for between 5 and 1000 significantly expressed genes")
    
    heatmap.error.flags <- which(c(class(try.data))=="try-error") #Qian 4/29/09
    #  if((class(try.data)!="try-error") && (nrow(expdata)<5 | nrow(expdata)>1000)) heatmap.error.flags <- c(heatmap.error.flags,5) #Qian 6/28/09
    
    if(length(heatmap.error.flags)==0){
      if(!is.na(match("Symbol",names(genelabs)))){
        gsymbols <- c()
        for(i in 1:nrow(genelabs)){
          sb <- genelabs$Symbol[i]
          slashmatches <- gregexpr(" /// ",sb)[[1]]
          if(length(slashmatches)>1) sb <- substr(genelabs$Symbol[i],1,slashmatches[2]-1)
          gsymbols <- append(gsymbols,sb)     #--> Shortens symbol listings - only displays first two
        }
      } else {
        gsymbols <- genelabs[,1]     #--> Just uses first column entries in the 'curgenelistfile' as labels if there is no 'Symbol' column in genelabs
      }
      # header.genecls <- scan(paste(WorkPath, "ExpDescWkSht.txt", sep="/"), what="", nlines=1, sep="\t", quote="") #Qian 4/29/09
      # ind.arrays <- !is.na(genecls[,match(ClassVariable, header.genecls)]) #Qian 4/29/09
      try.call <- try(CCHeatmap(xdata=expdata, #--> this data is ordered exactly as in the expression data table (rows: genes in increasing P-value, columns: arrays in alphabetical order by class)
                                geneidlabel=gsymbols,
                                classid=experiments$pheno, #--> ClassVariable defined in RunR.R; use 'gsub' to ensure ClassVariable finds a match in names(genecls): (whitespace replaced by '.' in read-in) #Qian 4/29/09
                                arrayid=experiments$ExpId, #--> reorder arrayid according to ExpId. #Qian 4/29/09
                                ExpId=experiments$ExpId,  #--> experiments Id, used as a base for reordering the arrays in heatmap
                                pairedid=experiments$id,  #Qian 8/30/2012 paired data show patient ID instead of array ID (such as Perou dataset).
                                isSingleChannel=IsSingleChannel, #--> IsSingleChannel defined in RunR.R
                                NumColors=50, #--> Hard-coded number and type of colors in heatmap
                                MinColor="green",
                                MidColor="black",
                                MaxColor="red",
                                dividers="yellow",
                                outputdir=paste(projectPath, "/Output/", AnalysisName, sep=""),
                                outpfilepng="heatmap.png",
                                outpfilehtml="heatmap.html",
                                paired=paired,
                                ArrayToolsPath=ArrayToolsPath) , #Qian 03/30/09 for paired
                      silent=TRUE)
      
      if(class(try.call)=="try-error") {
        HEAT.ERROR <- TRUE
        heatmap.error.text[6] <- geterrmessage()
        heatmap.error.flags <- c(heatmap.error.flags,6)
      } else {
        HEAT.ERROR <- FALSE
      }
    } else {
      HEAT.ERROR <- TRUE
    }
    ##***END HEATMAP CODE
  }
  
  CreateExpressionTable(projectPath=projectPath,
                        AnalysisName=AnalysisName,
                        AnalysisType=AnalysisType,
                        WorkPath=WorkPath,
                        ArrayToolsPath=ArrayToolsPath,
                        nGenesInTable=nrow(genelabs),
                        HEAT.ERROR=HEAT.ERROR,
                        heatmap.error.text=heatmap.error.text,
                        heatmap.error.flags=heatmap.error.flags,
                        RunLFDR=RunLFDR,
                        classifiers=classifiers,
                        codeReg=codeReg,
                        paired=paired)
  
  if (codeSurv==0) {
    #source(file.path(ArrayToolsPath, "Fortran/PCP.R"))
    currentPath = getwd()
    PCP(WorkPath=WorkPath,projectPath=projectPath,AnalysisName=AnalysisName,
        RunLFDR=RunLFDR,paired=paired,ClassVariableName=ClassVariableName,ind.deg.lfdr=ind.deg.lfdr)
    setwd(currentPath)
  }
  
  return(resList)
}
