# 4/18/17    Ting modified code to use EntrezID to generate KEGG pathway web links with map coloring (in the previous version, gene symbol was used)
# 8/11/2014  Ting added to support predicted TF target genelists 
# 06/30/09   Ming-Chung revert back not to output gene list for gene set analysis.
# 5/7/09     Qian modified to show 3 significant digits for FDR.
# 5/7/09     Qian modified to show 2 significant digits for Geometric means and fold changes.
# 07/15/07	 Yong modified to use Bioconductor annotation package for GO
# 06/10/07   Yong re-organized gene set comparison analysis
# 05/15/07   Deyun added to remove the file folder for HTML output
# 04/24/07	 Yong modified to add Gene Set Global Test using Bioconductor package: globaltest.
# 01/09/07   Deyun modified to add links for Broad pathway
# 01/04/07   Deyun Remove the file path for GeneListComparison
# 12/7/06    Yong	modified to replace dget/dput with load/save for R2.4.
# 12/29/06   Ming-Chung modified to add Survival-GO analysis
# 10/17/2006 Deyun added for protein domain analysis 
# 11/10/2006 Deyun added for transcription factor analysis 
# 12/11/2006 Deyun added for MicroRNA analysis 

##                                                                                                           ##
## THIS FILE CONTAINS ONE FUNCTION (GetGenesHTML) WHICH IS CALLED BY THE CLASS COMPARISON,                   ##
## CLASS PREDICTION, SURVIVAL, AND QUANTITATIVE TRAIT ANALYSES.  THE OUTPUT OF THIS FUNCTION                 ##
## IS A LIST CONTAINING STRINGS WHICH CAN BE INSERTED INTO HTML OUTPUT FILES TO CREATE                       ##
## THE TABLE OF SIGNIFICANT GENESETS AND TABLE OF GENES WITHIN THOSE GENESETS (FOR GENE SET                  ##
## COMPARISON) OR THE TABLE OF 'SIGNIFICANT GENES' (FOR ALL OTHER ANALYSIS TYPES), AND THE GENE              ##
## ONTOLOGY OBSERVED v. EXPECTED TABLE (IF APPLICABLE).  THE CHROMOSOMAL DISTRIBUTION BARPLOT                ##
## IS CREATED AS A SIDE EFFECT (BUT NOT A RETURNED OBJECT) OF THIS FUNCTION.                                 ##
##                                                                                                           ##
##  Notes:                                                                                                   ##
##  - 'classifiers' and 'GenesFiltered' are defined in <GetXXXXResults.R> files.                             ##
##  - 'AnalysisType', 'IsSingleChannel' defined in <RunXXXX.R> scripts.                  ##
##  - RunPermP' seems to be used only for 'ClassComparison', and is defined elsewhere ??                     ##
##  - codeSurv' is defined in 'GetComparisonResults.R' file.                                                 ##
##  - CVsupport', 'methods' and 'SameN' are defined in 'GetPredictionResults.R' file.                        ##
##  - SampleCode', 'SampleGroup' and 'GeneExpData' are defined in 'GetBinaryTreePredictionResults.R' file.   ##
##  - CVsupport', 'coxcoef' are defined in 'GetRiskResults.R' file.                                             ##
##   'ncomp' and 'pHotelling' required for Hotelling test are defined in 'RunGOComparison.R',                ##
##                 'GetGOcomparison.R' and other gene set comparison file.                                   ##
##  - 'StatusVariable' is defined in <RunXXXX.R> script.                                                     ##
##  - 'genepairIndex' is defined in 'GetPredictionResults.R> file.                                           ##
##  - 'xikprime' is defined in 'GetPAMResults.R> file.                                                      ##

GetGenesHTML <- function(
  index
  , classlabels=NULL
  , paired=FALSE
  , TruncLevel=10^(-7)
  , RefClass = NULL
  , univtest = FALSE
  , puniv = NULL
  , DoMasterAnnotations=TRUE
  , WorkPath
  , Hotelling = FALSE
  , GeneSetGlobalTest = FALSE
  , GeneSetGSA = FALSE
  , pGeneSet = NULL
  , npermGSA = NULL
  , GSAscore = NULL
  , GeneSetUnit = NULL
  , GeneTableCaption = NULL
  , classifiers
  , LogBase
  , projectPath
  , PAllgenes
  , RunLFDR
  , RunPermP
  , IsSingleChannel
  , AnalysisName
  , AnalysisType
  , DoGOOvE
) {
  
  Trunc <- function(x, cutoff=NULL, pos=TRUE, TruncLevel=10^(-7), npermGSA=NULL, 
                    GSAscore=NULL, classlabels=NULL) {
    # Unify the way a numerical number is printed out in a HTML table.
    # Try not to use round, signif individually in the code unless there is a reason.
    # Input:
    #   pos: is the value always positive? Useful for printing correlation, t-statistic and coefficients from a regression.
    TruncDigits <- as.integer(log(TruncLevel, 10)*(-1))
    if (is.null(cutoff)) {
      if (pos) {
        ifelse(x >= TruncLevel, round(x, TruncDigits), paste("<", TruncLevel)) 
      } else {
        round(x, 3) # should compare with the header 't-value' width. Too more can result in word wrap. 
      } 
    } else if (!is.null(npermGSA)) {
      options(scipen=4) #use fix notation for decimal value.
      TruncDigits <- 5
      TruncLevel <- 1/npermGSA  # override TruncLevel for GSA method!
      sgn <- if (length(unique(classlabels)) == 2 | exists("StatusVariable")) {
        ifelse(GSAscore >=0, "(+)", "(-)")
      } else rep("", length(x))
      ret.x <- ifelse(x >= TruncLevel,
                      ifelse(x <= cutoff, paste("<font color=\"red\">", round(x, TruncDigits), sgn, "</font>"),
                             paste(round(x, TruncDigits), sgn)),
                      paste("<font color=\"red\">", "<", round(TruncLevel, TruncDigits), sgn, "</font>"))
      options(scipen=0) # restore default
      return(ret.x)
    } else {
      options(scipen=4) #use fix notation for decimal value.
      ret.x <- ifelse(x >= TruncLevel,
                      ifelse(x < cutoff, paste("<font color=\"red\">", round(x, TruncDigits), "</font>"),
                             round(x, TruncDigits)),
                      paste("<font color=\"red\">", "<", TruncLevel, "</font>"))
      options(scipen=0) # restore default
      return(ret.x)
    }	
  }
  
  #Qian 5/7/09
  signifNew <- function(x, digits=3) {
    return(ifelse(x >= 0, 
                  ifelse(x >= 1, round(x, digits), ifelse(x >= 1e-7, signif(x, digits), "< 1e-07")),
                  ifelse(x <= -1, round(x, digits), ifelse(x <= -1e-7, signif(x, digits), "> -1e-07")))
    )
  }
  #EndQian
  
  ## SOURCE FUNCTIONS FROM PLUGINS BRBFUNS FOLDER.
  
  #source(paste(ArrayToolsPath, "\\R\\Misc.R", sep=""))
  #SourceFilesInFolder(paste(ArrayToolsPath, "\\plugins\\BRBfuns", sep=""), PrintErr=TRUE)
  
  ## DEFINE PARAMETERS FOR DIFFERENT ANALYSIS TYPES.
  
  GeneSetComparison <- FALSE
  HasClasses <- FALSE
  ValueDef <- ifelse(IsSingleChannel,"intensities","ratios")
  
  if (AnalysisType == "ClassComparison" | AnalysisType == "BlockClassComparison") {
    HasClasses <- TRUE
  }
  
  if (AnalysisType == "SAM" ) {
    #Note that SAM does not have the 'classifiers' object.
    RunPermP <- FALSE
    HasClasses <- TRUE
  }
  
  if (AnalysisType == "ClassPrediction" | AnalysisType == "BinaryTreePrediction" | AnalysisType == "PAM") {
    RunPermP <- FALSE
    HasClasses <- TRUE
    DoGOOvE <- FALSE; MinObserved <- MinRatio <- NA
  }
  
  if (AnalysisType == "GOComparison" | AnalysisType == "PathwayComparison" | AnalysisType == "GeneListComparison") {
    RunPermP <- FALSE
    GeneSetComparison <- TRUE
    HasClasses <- if (!exists("StatusVariable")) TRUE else FALSE
    DoGOOvE <- FALSE; MinObserved <- MinRatio <- NA
  }
  GONumTotCat <- GONumSigCat <- NULL
  
  if (AnalysisType == "SurvivalRisk") {
    paired <- FALSE
    univtest <- FALSE
    RunPermP <- FALSE
    DoGOOvE <- FALSE; MinObserved <- MinRatio <- NA
  }
  
  if (AnalysisType == "Survival") {
    paired <- FALSE
  }
  
  if (AnalysisType == "Correlation") {
    paired <- FALSE
    # RunPermP <- FALSE MLI 2/17/2010
  }
  
  ## CREATE CODE FOR THE HTML TABLE OF SIGNIFICANT GENE SETS, FOR GENE SET COMPARISON.
  
  if (GeneSetComparison) {
    
    # First, create a table of gene set categories ('GeneSetIndex' is the list of significant 
    # categories in the increasing p-value order.
    
    # NOTE: For Leonid's original code for case when AddGeomeanColumns=TRUE, see
    #       'PrepareGOComparisonTableHTML.R' file from v3.4.  AddGeomeanColumns has been
    #       hard-coded as FALSE for some time now, thus GeneSetTable2 is also no longer used.
    
    GeneSetIndex <- index
    
    # Get the R object with gene set categories as a list.
    load(paste(WorkPath, "/GeneSetList.rda", sep=""))
    GeneSetList.raw <- GeneSetList[GeneSetIndex] #note GeneSetList is the object name stored in GeneSetList.rda
    
    GeneSetList <- list(2) #note now GeneSetList is a list !
    
    for(i in 1:length(GeneSetList.raw) ) {
      GeneSetListElement <- strsplit(GeneSetList.raw[i],split=" ")[[1]]
      GeneSetList[[i]] <- as.double(GeneSetListElement[!is.element(GeneSetListElement,c(""," "))])
    }
    
    load(paste(WorkPath, "/GeneSetName.rda", sep=""))
    
    if (AnalysisType == "GOComparison") {
      GeneSetIdName <- "GO category"
      GeneSetId <- GeneSetName[GeneSetIndex]
      GeneSetnames <- GeneSetId
      GeneSetCode <- GeneSetId
      load(paste(WorkPath, "/GODesc.rda", sep=""))
      GOOntologyName <- "GO ontology"
      GOOntologySig <- GOOntology[match(GeneSetnames, names(GOOntology))]
      GeneSetDescriptionName <- "GO term"
      GeneSetdescriptions <- GOTerm[match(GeneSetnames, names(GOTerm))]
      GONumTotCat <- c(length(which(GOOntology=="CC")),length(which(GOOntology=="MF")),length(which(GOOntology=="BP")))
      GONumSigCat <- c(length(which(GOOntologySig=="CC")),length(which(GOOntologySig=="MF")),length(which(GOOntologySig=="BP")))
      
      #2/20/08 create GO tree graph
      #cat('Generating GO tree graph ...\n', file=stderr())
      #HasGOTree <- TRUE
      #GOTreeSetIds <- GeneSetName[GOTreeIndex]
      #GOTreeSet <- GOOntology[match(GOTreeSetIds, names(GOOntology))]
      #GOBPGeneSets <- names(which(GOTreeSet=="BP"))
      #GOMFGeneSets <- names(which(GOTreeSet=="MF"))
      #GOCCGeneSets <- names(which(GOTreeSet=="CC"))      
      
      #if (!require(GO) || !require(Rgraphviz) || !require(GOstats)) {
      # source(paste(ArrayToolsPath,"/R/BiocUtil.R", sep=""), echo = FALSE)
      #        if (!LoadRPackage("GOstats", isBioconductor=TRUE)) HasGOTree<-FALSE
      #        if (!LoadRPackage("GO", isBioconductor=TRUE)) HasGOTree<-FALSE
      #        if (!LoadRPackage("Rgraphviz", isBioconductor=TRUE)) HasGOTree<-FALSE
      #      }
      
      #if (HasGOTree) {
      #        require(Rgraphviz)        
      #        #set image size
      #        gWidth.large = 2400
      #        gHeight.large = 3000
      #        gWidth.small = 200
      #        gHeight.small = 250
      #
      #        if(length(GOBPGeneSets)>0) {
      #          gBP <- GOGraph(GOBPGeneSets, GOBPPARENTS)
      #          #gBPlo <- agopen(gBP, "gBP")
      #          nBP = list()
      #          lbs = nodes(gBP)
      #          names(lbs) = nodes(gBP)
      #          nBP$label = lbs
      #          ndcolors = rep("transparent", length(nodes(gBP)))
      #          names(ndcolors) = nodes(gBP)
      #          ndcolors[names(ndcolors) %in% GOBPGeneSets] = "yellow"          
      #          nBP$fillcolor = ndcolors
      #          png(paste(projectPath, "/Output/", AnalysisName, "/GOBPlarge.png", sep=""), width=gWidth.large, height=gHeight.large)
      #          #plot(gBPlo, nodeAttrs=nBP, main="Biological Process")
      #          plot(gBP, nodeAttrs=nBP, main="Biological Process")
      #          dev.off()
      #          png(paste(projectPath, "/Output/", AnalysisName, "/GOBPsmall.png", sep=""), width=gWidth.small, height=gHeight.small)
      #          plot(gBP, nodeAttrs=nBP, main="Biological Process")
      #          dev.off()
      #        }
      #
      #        if(length(GOMFGeneSets)>0) {
      #          gMF <- GOGraph(GOMFGeneSets, GOMFPARENTS)
      #          #gMFlo <- agopen(gMF, "gMF")
      #          nMF = list()
      #          lbs = nodes(gMF)
      #          names(lbs) = nodes(gMF)
      #          nMF$label = lbs
      #          ndcolors = rep("transparent", length(nodes(gMF)))
      #          names(ndcolors) = nodes(gMF)
      #          ndcolors[names(ndcolors) %in% GOMFGeneSets] = "yellow"          
      #          nMF$fillcolor = ndcolors
      #          png(paste(projectPath, "/Output/", AnalysisName, "/GOMFlarge.png", sep=""), width=gWidth.large, height=gHeight.large)
      #          #plot(gMFlo, nodeAttrs=nMF, main="Molecular Function")
      #          plot(gMF, nodeAttrs=nMF, main="Molecular Function")
      #          dev.off()
      #          png(paste(projectPath, "/Output/", AnalysisName, "/GOMFsmall.png", sep=""), width=gWidth.small, height=gHeight.small)
      #          plot(gMF, nodeAttrs=nMF, main="Molecular Function")
      #          dev.off()
      #        }
      #        
      #        if(length(GOCCGeneSets)>0) {
      #          gCC <- GOGraph(GOCCGeneSets, GOCCPARENTS)      
      #          #gCClo <- agopen(gCC, "gCC")
      #          nCC = list()
      #          lbs = nodes(gCC)
      #          names(lbs) = nodes(gCC)
      #          nCC$label = lbs
      #          ndcolors = rep("transparent", length(nodes(gCC)))
      #          names(ndcolors) = nodes(gCC)
      #          ndcolors[names(ndcolors) %in% GOCCGeneSets] = "yellow"          
      #          nCC$fillcolor = ndcolors
      #          png(paste(projectPath, "/Output/", AnalysisName, "/GOCClarge.png", sep=""), width=gWidth.large, height=gHeight.large)
      #          #plot(gCClo, nodeAttrs=nCC, main="Cellular Component")
      #          plot(gCC, nodeAttrs=nCC, main="Cellular Component")
      #          dev.off()
      #          png(paste(projectPath, "/Output/", AnalysisName, "/GOCCsmall.png", sep=""), width=gWidth.small, height=gHeight.small)
      #          plot(gCC, nodeAttrs=nCC, main="Cellular Component")
      #          dev.off()
      #        }
      #      }
    } else {
      GOOntologyName <- NULL
      GOOntologySig <- NULL
    }
    
    if (AnalysisType == "PathwayComparison") {
      GeneSetId <- GeneSetName[GeneSetIndex]
      GeneSetnames <- GeneSetId
      GeneSetCode <- GeneSetId
      GeneSetIdName <- paste(PathwayType, "Pathway")
      GeneSetDescriptionName <- "Pathway description"
      if (PathwayType == "Biocarta") {
        BiocartaTXT <- read.table(paste(ArrayToolsPath, "/Pathways/Output/Names_Biocarta.txt", sep=""), sep="\t", header=FALSE)
        GeneSetdescriptions <- as.character(BiocartaTXT[[2]]) [match(GeneSetnames,as.character(BiocartaTXT[[1]]))]
        GeneSetdescriptions <- paste("<A HREF=\"http://cgap.nci.nih.gov/Pathways/BioCarta/", GeneSetId, "\">", GeneSetdescriptions, "</A>", sep="")
      }
      if (PathwayType == "Kegg") {
        KeggTXT <- read.table(paste(ArrayToolsPath, "/Pathways/Output/Names_KEGG.txt", sep=""), sep="\t", header=FALSE)
        GeneSetdescriptions <- as.character(KeggTXT[[2]]) [match(GeneSetnames,as.character(KeggTXT[[1]]))]
        #GeneSetdescriptions <- paste("<A HREF=\"http://cgap.nci.nih.gov/Pathways/Kegg/", GeneSetId, "\">", GeneSetdescriptions, "</A>", sep="")
        GeneSetdescriptions <- paste("<A HREF=\"http://www.kegg.jp/kegg-bin/show_pathway?", GeneSetId, "\">", GeneSetdescriptions, "</A>", sep="")
      }
    }
    
    if (AnalysisType == "GeneListComparison") {
      
      GeneSetName <- unlist(lapply(strsplit(GeneSetName,"/"), function(x) rev(x)[1]))
      GeneSetName <- unlist(strsplit(GeneSetName,".txt"))
      GeneSetnames <- GeneSetName[GeneSetIndex]
      
      if (GeneSetGroup == "TranscriptionFactor") {
        GeneSetCode <- unlist(lapply(strsplit(GeneSetnames, "_"), function(x) rev(x)[1]))
        GeneSetCode <- gsub(" ", "", GeneSetCode)
        GeneSetnames <- paste("<a href=\"http://www.gene-regulation.com/cgi-bin/pub/databases/transfac/getTF.cgi?AC=", GeneSetCode, "\">", GeneSetnames, "</a>", sep="")
      }
      
      # Ting added code to support predicted TF target genelists 8/11/2014
      if (GeneSetGroup == "TranscriptionFactorPredict") {
        GeneSetCode <- unlist(lapply(strsplit(GeneSetnames, "_"), function(x) rev(x)[1]))
        GeneSetCode <- gsub(" ", "", GeneSetCode)
        GeneSetnames <- paste("<a href=\"http://jaspar.genereg.net/cgi-bin/jaspar_db.pl?ID=", GeneSetCode, "&rm=present&collection=CORE", "\">", GeneSetnames, "</a>",sep = "")
      }
      
      if (GeneSetGroup == "MicroRNA") {
        GeneSetCode <- unlist(lapply(strsplit(GeneSetnames, "_"), function(x) rev(x)[1]))
        GeneSetCode <- gsub(" ", "", GeneSetCode)
        GeneSetnames <- paste("<a href=\"http://microrna.sanger.ac.uk/cgi-bin/sequences/query.pl?terms=", GeneSetCode, "\">", GeneSetnames, "</a>", sep="")
      }
      if (GeneSetGroup == "ProteinDomain") {
        GeneSetCode <- unlist(lapply(strsplit(GeneSetnames, "_"), function(x) x[1]))
        GeneSetCode <- gsub(" ", "", GeneSetCode)
        if (tolower(substr(GeneSetCode[1],1,2))=="pf") 
          GeneSetnames <- paste("<a href=\"http://pfam.sanger.ac.uk/family?acc=", GeneSetCode, "\">", GeneSetnames, "</a>", sep="")
        if (tolower(substr(GeneSetCode[1],1,2))=="sm") 
          GeneSetnames <- paste("<a href=\"http://smart.embl.de/smart/do_annotation.pl?DOMAIN=", GeneSetCode, "\">", GeneSetnames, "</a>", sep="")          
      }
      if (GeneSetGroup == "Broad") {
        GeneSetCode <- GeneSetnames
        GeneSetnames <- paste("<a href=\"http://www.broad.mit.edu/gsea/msigdb/cards/", GeneSetnames, ".html\"  TARGET=_blank>", GeneSetnames, "</a>", sep="")
      }
      #Qian 3/13/09
      if (GeneSetGroup == "Lymphoid") {
        GeneSetCode <- GeneSetnames
        GeneSetnames <- paste("<a href=\"http://lymphochip.nih.gov/cgi-bin/signaturedb/sig_id_query.cgi?Signature=", GeneSetCode, "\">", GeneSetnames, "</a>", sep="")
      }
      #EndQian
      if (GeneSetGroup == "GeneList") {
        GeneSetCode <- GeneSetnames
      }
      
      GeneSetId <- GeneSetnames
      GeneSetIdName <- paste(GeneSetGroup, " GeneSets")
      GeneSetdescriptions <- NULL
      GeneSetDescriptionName <- NULL
    }
    
    n <- length(GeneSetIndex)
    order.pvalue <- order(pGeneSet$pLS)
    pGeneSet <- pGeneSet[order.pvalue,,drop=FALSE]
    GSAscore <- GSAscore[order.pvalue] # MLI 11/10/2008
    
    GeneSetIndex <- GeneSetIndex[order.pvalue]
    GeneSetList <- GeneSetList[order.pvalue]
    GeneSetId <- GeneSetId[order.pvalue] 
    GeneSetnames <- GeneSetnames[order.pvalue] 
    GeneSetdescriptions <- GeneSetdescriptions[order.pvalue]
    GeneSetCode <- GeneSetCode[order.pvalue]
    if (AnalysisType == "GOComparison") { GOOntologySig  <- GOOntologySig[order.pvalue] }
    
    # Compute number of genes in the gene set catagory:
    GeneTableFileName <- "GeneSetGenesTable"
    if (HasClasses) {
      NgenesGeneSet <- rep("", n)
      HeatmapGeneSet <- rep("", n)
      for(i in 1:n) {
        #6/8/07 Yong modified to separate gene tables out from gene-set table html file.
        #NgenesGeneSet[i] <- paste("<a href=#", GeneSetCode[i], ">", length(GeneSetList[[i]]), "</a>", sep="")
        FileNum <- ((i-1) %/% GeneSetUnit + 1)
        NgenesGeneSet[i] <- paste("<a href=\"", GeneTableFileName, FileNum, ".html#", GeneSetCode[i], "\">", length(GeneSetList[[i]]), "</a>", sep="")
        #add gene set heatmap link 2/14/08
        HeatmapGeneSet[i] <- paste("<a href=\"", "heatmap/geneset", i, ".html", "\" TARGET=_blank>", "heatmap", "</a>", sep="")
      }
    } else {
      NgenesGeneSet <- rep("", n)
      for(i in 1:n) {
        FileNum <- ((i-1) %/% GeneSetUnit + 1)
        NgenesGeneSet[i] <- paste("<a href=\"", GeneTableFileName, FileNum, ".html#", GeneSetCode[i], "\">", length(GeneSetList[[i]]), "</a>", sep="")
      }      
    }
    # Create HTML table of gene set categories.
    fillNULL <- NULL
    DoKStest <- "pKS" %in% colnames(pGeneSet)
    GeneSetTable <- cbind(GeneSetId, GOOntologySig, GeneSetdescriptions, NgenesGeneSet, if (HasClasses) HeatmapGeneSet,
                          Trunc(pGeneSet$pLS, cutoff=alpha, TruncLevel=10^(-5)),
                          if(DoKStest) Trunc(pGeneSet$pKS, cutoff=alpha, TruncLevel=10^(-5)),
                          if(!GeneSetGSA) NULL else Trunc(pGeneSet$pGSA, cutoff=alpha, npermGSA=npermGSA, GSAscore=GSAscore, classlabels=classlabels),                      
                          if(!Hotelling) NULL else Trunc(pGeneSet$pHotelling, cutoff=alpha),
                          if(!GeneSetGlobalTest) NULL else Trunc(pGeneSet$pGlobal, cutoff=alpha))
    
    GeneSetTableNames <- c(GeneSetIdName, GOOntologyName, GeneSetDescriptionName, "Number of <NOBR>genes</NOBR>", if (HasClasses) "Heatmap link",
                           "LS permutation <NOBR>p-value</NOBR>", 
                           if(DoKStest) "KS permutation <NOBR>p-value</NOBR>",
                           if(!GeneSetGSA) NULL else "Efron-Tibshirani's GSA test <NOBR>p-value</NORB>",
                           if(!Hotelling) NULL else "Hotelling test <NOBR>p-value</NOBR>",
                           if(!GeneSetGlobalTest) NULL else "Goeman's global test <NOBR>p-value</NOBR>")
    
    dimnames(GeneSetTable)[[2]] <- GeneSetTableNames
    GeneSetTable1 <- CreateHtmlTable(GeneSetTable)
    
    # Leonid's original code created GeneSetTable2 only when data is paired AND AddGeomeanColumns=TRUE.
    GeneSetTable2 <- NULL
    
    # Create index of genes in gene set categories
    # sort genes in the gene table by the pvalues in each gene set
    # and also define 'Pvalues' as the ordered pvalues for signficiant genes.
    AllPvalues <- read.table(paste(WorkPath,"/Pvalues.txt", sep=""), header=TRUE)[,2]
    GeneTablePartition <- rep(0, (n %/% GeneSetUnit + 1))
    GeneRowNum <- c(0)
    prevTableNum <- TableNum <- 0
    for(i in 1:n) {
      prevTableNum <- TableNum
      TableNum <- i %/% GeneSetUnit
      if(i == 1) {
        tmp <- GeneSetList[[i]]
        index <- tmp[order(AllPvalues[tmp])]
        genesGeneSetnames <- c(paste("<a name=", GeneSetCode[i], "></a>", GeneSetnames[i]), rep("", length(GeneSetList[[i]])-1) )
        genesGOTerms <- rep(GOOntologySig[i],length(GeneSetList[[i]]) )
        genesGeneSetdescriptions <- rep(GeneSetdescriptions[i],length(GeneSetList[[i]]) )
        GeneRowNum <- rep(1:length(GeneSetList[[i]]))
      } else { 
        tmp <- GeneSetList[[i]]
        index <- c(index, tmp[order(AllPvalues[tmp])])
        genesGeneSetnames <- c(genesGeneSetnames,c(paste("<a name=", GeneSetCode[i], "></a>", GeneSetnames[i]),rep("", length(GeneSetList[[i]])-1) ))
        genesGOTerms <- c(genesGOTerms,rep(GOOntologySig[i],length(GeneSetList[[i]]) ))
        genesGeneSetdescriptions <- c(genesGeneSetdescriptions,rep(GeneSetdescriptions[i],length(GeneSetList[[i]]) ))
        GeneRowNum <- c(GeneRowNum, rep(1:length(GeneSetList[[i]])))
      }
      if (TableNum > prevTableNum)  GeneTablePartition[TableNum] <- length(index)       
    }
    GeneTablePartition[TableNum + 1] <- length(index)
    Pvalues <- AllPvalues[index]
  } else {  # if not GeneSetComparison.
    GeneSetTable1 <- NULL
    GeneSetTable2 <- NULL
  }
  
  ## THE REST OF THIS CODE CREATES CODE FOR THE HTML TABLE OF SIGNIFICANT GENES
  ## (OR GENES CONTAINED WITHIN SIGNIFICANT GENE SETS, FOR GENE SET COMPARISON).
  
  ## FOR BINARY TREE PREDICTION, CREATE GENE EXPRESSION DATA TABLE FOR EACH SUBTREE ???
  
  if (AnalysisType == "BinaryTreePrediction") {
    GeneExpSampleCode <- as.double(unlist(GeneExpData[1,])[-1])
    GeneExpGroupCode <- SampleCode[GeneExpSampleCode]
    GeneExpData <- GeneExpData[-1,]
    if( any(index != GeneExpData[[1]]) ) stop("Incorrect input: index != GeneExpData[[1]] ")
    GeneExpData <- GeneExpData[,-1]
  }
  
  ## READ GENE ID TABLE FROM FILES.
  
  load(paste(WorkPath, "/GeneIdNames.rda", sep=""))
  NumGeneIds <- length(GeneIdNames)
  load(paste(WorkPath, "/GeneId1.rda", sep=""))
  NumGenesInAnalysis <- length(GeneId1)
  GeneId <- matrix("", nrow=NumGenesInAnalysis, ncol=NumGeneIds)
  GeneId[,1] <- GeneId1
  GeneId[,1] <- gsub("@#$%&", "'", GeneId[,1], fixed=TRUE)
  GeneId[,1] <- gsub("@#$%!", "\"", GeneId[,1], fixed=TRUE)
  if (NumGeneIds > 1) {
    for (i in 2:NumGeneIds) {
      load(paste(WorkPath, "/GeneId", i, ".rda", sep=""))
      GeneId[,i] <- get(paste("GeneId",i,sep=""))
      GeneId[,i] <- gsub("@#$%&", "'", GeneId[,i], fixed=TRUE)
      GeneId[,i] <- gsub("@#$%!", "\"", GeneId[,i], fixed=TRUE)
    }
  }
  
  #1/22/08. added modification adjusted for the gene probe to gene symbol reduction method used for gene set analysis
  #modify since the probe reduction is moved to the main dialog, MLI 1/11/2010
  #if (GeneSetComparison) {
  #  PrimaryIDFiltered <- GeneId1[,1] #GeneId1 is a matrix
  #  FilteredIndex <- as.numeric(names(PrimaryIDFiltered)) #the index is row number of the full set of genes
  #  load(paste(WorkPath, "/ProbeReductionIndex.rda", sep=""))
  #  NumGenesInAnalysis <- length(ProbeReductionIndex)
  #  GeneId <- GeneId[FilteredIndex %in% ProbeReductionIndex,,drop=FALSE]
  #}
  
  ### Supriya commented removing the "Defined genelist" column out for v3.5 as a separate analysis output folder is created.
  
  # Remove "Defined genelists" column from the GeneId
  #if (length(colrm <- which(GeneIdNames == "Defined genelists")) > 0) {
  #  GeneId <- GeneId[,-colrm]
  #  NumGeneIds <- NumGeneIds - 1
  #  GeneIdNames <- GeneIdNames[-colrm]
  #}
  
  
  ### MLI added for pairwise class comparison 12/24/08
  GeneId <- GeneId[index,,drop=FALSE ]
  if (file.exists(paste(WorkPath, "/pairwiseCC.rda", sep=""))) {
    load(paste(WorkPath, "/pairwiseCC.rda", sep=""))
    GeneId <- cbind(GeneId, sapply(pairwiseCC, function(x) paste(x, collapse=", ")))
    GeneIdNames <- c(GeneIdNames, "Pairwise significant")     
  }
  
  if (AnalysisType == "Survival") {
    # add KM curve hyperlinks to the end of GeneId
    GeneId <- cbind(GeneId, paste("<a href=kmcurve/gene", 1:length(index), ".html>Curve</a>", sep=""))
    GeneIdNames <- c(GeneIdNames, "Kaplan-Meier-<BR>curve")               
    NumGeneIds <- NumGeneIds + 1    
  }
  
  ## IDENTIFY PROBESETS COLUMN, IF ANY.
  
  ProbeSets <- NA
  GeneIdColumn <- match("Probe set", GeneIdNames, nomatch=NA)
  if (!is.na(GeneIdColumn)) ProbeSets <- GeneId[,GeneIdColumn]
  
  ## DETERMINE NUMBER OF CLASSES, IF APPLICABLE.
  
  if (HasClasses) {
    nclasses <- if (paired) 2 else length(unique(classlabels))
  }
  
  ## SORT GENE ID TABLE (BY P- OR T-VALUES).
  
  ordercode <- NULL
  
  if ((AnalysisType != "ClassPrediction") & (AnalysisType != "SAM") & (AnalysisType != "PAM") & (AnalysisType != "SurvivalRisk") & (!GeneSetComparison)) {
    ordercode <- order(classifiers$pvalue)
    if (length(index) > 1) {
      GeneId <- GeneId[ordercode,,drop=FALSE]
      classifiers <- classifiers[ordercode,]
    }
  } else if (GeneSetComparison) {
    ordercode <- 1:length(index)
  }
  if (AnalysisType == "SurvivalRisk") {
    if (!uselasso) {
      ordercode <- order(puniv)
      if (length(index) > 1) {
        GeneId <- GeneId[ordercode,,drop=FALSE]
        puniv <- puniv[ordercode]
        CVsupport <- CVsupport[ordercode]
      }
    }
  }
  
  if (AnalysisType == "ClassPrediction") {
    if(nclasses == 2 & SameN) {
      ordercode  <-  order(classifiers$t)
    } else {
      ordercode  <-  order(classifiers$pvalue)
    }
    if (length(index) > 1) {
      GeneId <- GeneId[ordercode,,drop=FALSE]
      classifiers <- classifiers[ordercode,]
    }
  }
  
  ## READ OTHER DATA COLUMNS FROM FILES.
  
  if (HasClasses) {
    if (paired) {
      load(paste(WorkPath, "/meandiff.rda", sep=""))
      meandiff <- meandiff[index]
      meandiff <- LogBase^meandiff
      if (AnalysisType != "SAM") meandiff <- meandiff[ordercode]
    } else {
      if (AnalysisType == "BinaryTreePrediction") {
        #if(IsSingleChannel) {
        #  load(paste(WorkPath, "/medianLR.rda", sep=""))
        #  medianLR <- medianLR[index]
        #} else {
        #  medianLR <- rep(0,length(GeneExpData[[1]]))
        #}
        #medianLR <- medianLR[ordercode]
        for( i in 1:2) {
          temp <- GeneExpData[,GeneExpGroupCode==i]
          if(any(temp < -9999998)) temp[temp < -9999998] <- NA
          #meanLR <- LogBase^(rowMeans(temp, na.rm=TRUE)+medianLR)
          meanLR <- LogBase^(rowMeans(temp, na.rm=TRUE))
          if(i == 1) {meanLRall <- meanLR} else { meanLRall <- cbind(meanLRall,meanLR)}
        }
      } else {
        for( i in 1:nclasses) {
          load(paste(WorkPath, "/meanLR",i,".rda", sep=""))
          meanLR <- LogBase^meanLR[index]
          if(i == 1) {meanLRall <- meanLR} else { meanLRall <- cbind(meanLRall,meanLR)}
        }
      }
      if (length(ordercode)!=0) {
        meanLRall <- meanLRall[ordercode,,drop=FALSE]
      }
    }
  }
  ## FOR CLASS PREDICTION PROCEDURES, CREATE CV SUPPORT COLUMNS.
  
  if (AnalysisType == "ClassPrediction") {
    nMethods <- length(methods[methods==1])
    CVsupport.selected <- round(100*CVsupport[,c(methods==1),drop=FALSE])
    SameSupport <- TRUE
    if(nMethods > 1) {
      for(i in 1:nMethods) {
        SameSupport <- SameSupport & all(CVsupport[,1] == CVsupport[,i])
      }
    }
    if (length(index) > 1) {
      CVsupport   <- CVsupport[ordercode,]
      CVsupport.selected   <- CVsupport.selected[ordercode,]
    }
    if(SameSupport) {
      CVsupport.selected <- CVsupport.selected[[1]]
      CVsupport.names <- "% CV <NOBR>support</NOBR>"
    } else {
      CVsupport.names <- c("% CV <NOBR>support</NOBR><NOBR>CCP</NOBR>",
                           "% CV <NOBR>support</NOBR><NOBR>LDA</NOBR>",
                           "% CV <NOBR>support</NOBR><NOBR>K=1</NOBR>",
                           "% CV <NOBR>support</NOBR><NOBR>K=3</NOBR>",
                           "% CV <NOBR>support</NOBR><NOBR>Centroid</NOBR>",
                           "% CV <NOBR>support</NOBR><NOBR>SVM</NOBR>",
                           "% CV <NOBR>support</NOBR><NOBR>BCPP</NOBR>")
      CVsupport.names <- CVsupport.names[methods==1]
    }
  }
  
  if (AnalysisType == "BinaryTreePrediction") {
    CVsupport.selected <- round(100*classifiers$CVsupport,0)
    CVsupport.names <- "% CV <NOBR>support</NOBR>"
    CVsupport.selected <- CVsupport.selected[ordercode]
  }
  
  ## ADD GENE SET INFORMATION TO 'GeneId' TABLE, FOR GENE SET COMPARISON.
  
  ## NOTE: If the number of gene set column(s) appended before the unique id in 'GeneId' is changed,
  ##       then the GeneIdColumnToMatch parameter should also be modified correspondingly.
  
  if (GeneSetComparison) {
    if (AnalysisType == "GeneListComparison") {
      GeneId <- cbind(genesGeneSetnames,GeneId)
      NumGeneIds <- NumGeneIds + 1
      GeneIdNames <- c(GeneSetIdName, GeneIdNames)
      GeneIdColumnToMatch <- 2
    } else {
      GeneId <- cbind(genesGeneSetnames, genesGOTerms, genesGeneSetdescriptions, GeneId)
      if (is.null(genesGOTerms)) {
        NumGeneIds <- NumGeneIds + 2
      } else {
        NumGeneIds <- NumGeneIds + 3
      }
      GeneIdNames <- c(GeneSetIdName, GOOntologyName, GeneSetDescriptionName, GeneIdNames)
      if (is.null(genesGOTerms)) {
        GeneIdColumnToMatch <- 3
      } else {
        GeneIdColumnToMatch <- 4
      }
    }
  }
  
  ## DETERMINE n (=NUMBER OF GENES).
  
  if (length(index) > 1) {
    n <- if (is.matrix(GeneId)) nrow(GeneId) else length(GeneId)
  } else {
    n <- 1
  }
  
  ## CONCATENATE COLUMNS THAT BELONG IN HTML TABLE OF SIGNIFICANT GENES.
  ## NOTE: ALL COLUMNS SHOULD ALREADY BE SORTED BY 'ordercode' EXCEPT SAM, PAM AND GENESET COMPARISON.
  
  GeneIdTemp <- GeneIdTempPair <- NULL
  GeneIdNamesTemp <- GeneIdNamesTempPair <- NULL
  
  if (AnalysisType == "Correlation") {
    GeneIdTemp <- cbind(GeneIdTemp, Trunc(classifiers$correlation, pos=FALSE, TruncLevel=TruncLevel))
    GeneIdNamesTemp <- c(GeneIdNamesTemp, "Correlation <NOBR>coefficient</NOBR>")
  }
  
  if ((AnalysisType != "SAM") & (AnalysisType != "PAM") & (AnalysisType != "SurvivalRisk")) {
    if (!GeneSetComparison) Pvalues <- classifiers$pvalue
    GeneIdTemp <- cbind(GeneIdTemp, Trunc(Pvalues, TruncLevel=TruncLevel))
    GeneIdNamesTemp <- c(GeneIdNamesTemp, "Parametric <NOBR>p-value</NOBR>")
  }
  
  if (AnalysisType == "SAM") {
    # MLI added for v4.2 10/18/2010
    GeneIdTemp <- cbind(GeneIdTemp, Trunc(classifiers$di, pos=FALSE, TruncLevel=TruncLevel))
    GeneIdNamesTemp <- c(GeneIdNamesTemp, "d(i)")
  }
  
  if (AnalysisType == "ClassPrediction" | AnalysisType == "BinaryTreePrediction") {
    GeneIdTemp <- cbind(GeneIdTemp, Trunc(classifiers$t, pos=FALSE, TruncLevel=TruncLevel), CVsupport.selected)
    GeneIdNamesTemp <- c(GeneIdNamesTemp, "t-value", CVsupport.names)
  }
  
  if (AnalysisType == "SurvivalRisk") {
    if (!uselasso) {
      GeneIdTemp <- cbind(GeneIdTemp, Trunc(puniv, TruncLevel=TruncLevel), round(CVsupport,2))
      GeneIdNamesTemp <- c(GeneIdNamesTemp, "p-value", "% CV Support")
    } else {
      GeneIdTemp <- cbind(GeneIdTemp, Trunc(coxcoef, pos=FALSE, TruncLevel=10^(-5)), round(CVsupport,2))
      GeneIdNamesTemp <- c(GeneIdNamesTemp, "Coefficient", "% CV Support")
    }
  }
  
  if (univtest) {
    longp <- PAllgenes
    fdr <- p.adjust(longp, "fdr")[classifiers$Index]
    GeneIdTemp <- cbind(GeneIdTemp, signifNew(fdr, digits=3)) #Qian 5/7/09
    GeneIdNamesTemp <- c(GeneIdNamesTemp,"FDR")
  } 
  if (AnalysisType == "ClassComparison" && RunLFDR) {
    GeneIdTemp <- cbind(GeneIdTemp, signifNew(classifiers$lfdr, digits=3)) #MLI 2/19/2015
    GeneIdNamesTemp <- c(GeneIdNamesTemp,"<nobr>Local FDR</nobr>")
  } 
  
  if(RunPermP) {
    GeneIdTemp <- cbind(GeneIdTemp, Trunc(classifiers$permpvalue, TruncLevel=TruncLevel))
    GeneIdNamesTemp  <- c(GeneIdNamesTemp,"Permutation <NOBR>p-value</NOBR>") 
  }
  
  if (HasClasses) {
    if (paired) {
      DoSwap <- TRUE
      if(is.null(RefClass)) {
        DoSwap <- FALSE
      } else {
        if(as.character(RefClass) == as.character(classlabels[2]) ) {
          DoSwap <- FALSE
        }
      }
      if(DoSwap) {
        GeneIdTemp  <- cbind(GeneIdTemp, signifNew(1/meandiff, digits=2)) #Qian 5/7/09
        GeneIdNamesTemp <- c(GeneIdNamesTemp,paste("Geometric mean of ",ValueDef,"<BR>(class ",classlabels[2],
                                                   "/class ",classlabels[1],")",collapse=""))
      } else {
        GeneIdTemp  <- cbind(GeneIdTemp, signifNew(meandiff, digits=2)) #Qian 5/7/09
        GeneIdNamesTemp <- c(GeneIdNamesTemp,paste("Geometric mean of ",ValueDef,"<BR>(class ",classlabels[1],
                                                   "/class ",classlabels[2],")",collapse=""))
      }
    } else {
      if (AnalysisType == "BinaryTreePrediction") {
        for(i in 1:2) {  #Print geometric means for the two branches of the node.
          GeneIdTemp <- cbind(GeneIdTemp, signifNew(meanLRall[,i], digits=2)) #Qian 5/7/09
          GeneIdNamesTemp <- c(GeneIdNamesTemp,paste("Geom mean of ",ValueDef," in group ", i, collapse=""))
        }
      } else {
        for(i in 1:nclasses) {
          GeneIdTemp <- cbind(GeneIdTemp, signifNew(meanLRall[,i], digits=2)) #Qian 5/7/09
          GeneIdNamesTemp <- c(GeneIdNamesTemp,paste("Geom mean of ",ValueDef," in class ", i, collapse="")) #modified by Yong 12/15/06
        }
      }
      if(nclasses==2) {
        GeneIdTemp <- cbind(GeneIdTemp, signifNew(meanLRall[,1]/meanLRall[,2], digits=2)) #Qian 5/7/09
        GeneIdNamesTemp <- c(GeneIdNamesTemp,"Fold-change") # changed from "Ratio of geom means" 1/15/08
      }
      if (AnalysisType == "PAM") { # Ming-chung 8/5/2011
        for(i in 1:nclasses) {
          GeneIdTemp <- cbind(GeneIdTemp, 
                              ifelse(round(xikprime[,i], 2)==0, format(xikprime[, i], digits=3), round(xikprime[,i], 2)))
          GeneIdNamesTemp <- c(GeneIdNamesTemp, paste("Shrunken centroid in class ", i, collapse=""))
        }
        GeneIdTemp <- cbind(GeneIdTemp, signifNew(si, digits=2))
        GeneIdNamesTemp <- c(GeneIdNamesTemp, "Standard deviation s<sub>i</sub>+s<sub>0</sub>")
      }
    }
    #Not sure why this is necesary, but Amy copied it over from original PreparePredictionTableHTML.R file when merging.  6/29/06.
    if (AnalysisType == "ClassPrediction") {for(ii in 1:length(GeneId)) GeneId[[ii]] <- as.character(GeneId[[ii]])}
  }
  
  if (AnalysisType == "Survival") {
    LogTruncLevel <- log(TruncLevel)
    HazardRatio <- classifiers$b
    HazardRatio <- ifelse(HazardRatio < LogTruncLevel,paste("< ",TruncLevel),
                          ifelse(HazardRatio > -LogTruncLevel,paste("> ",1/TruncLevel),
                                 round(exp(HazardRatio),3) )) 
    GeneIdTemp <- cbind(GeneIdTemp, HazardRatio, round(classifiers$SD,3))
    GeneIdNamesTemp  <- c(GeneIdNamesTemp,"Hazard <NOBR>Ratio</NOBR>",paste("SD of log<NOBR>",ValueDef,"</NOBR>"))
  }
  
  if (GeneSetComparison) {
    GeneId  <- cbind(GeneId, GeneIdTemp)
    GeneIdNamesTemp <- c(GeneIdNames, GeneIdNamesTemp)
  } else {
    GeneId  <- cbind(GeneIdTemp, GeneId)
    GeneIdNamesTemp <- c(GeneIdNamesTemp, GeneIdNames)
    GeneIdColumnToMatch <- ncol(GeneIdTemp) + 1
  }
  
  GeneId <- as.data.frame(GeneId)
  names(GeneId) <- GeneIdNamesTemp      
  
  ## CALL FUNCTION TO CREATE HTML TABLE OF SIGNIFICANT GENES (INCLUDING "INFO" LINK),
  ## CHROMOSOMAL DISTRIBUTION BARPLOT, AND GO ANALYSIS (IF APPLICABLE).
  
  #6/8/07 Yong modified to probihit showing Chromsome bar plot when doing gene set comparison.
  #RunChromDistBarPlot <- if (Hotelling) FALSE else TRUE
  RunChromDistBarPlot <- if (GeneSetComparison) FALSE else TRUE
  GeneId1B <- GeneId1 # debug use
  TableID <- if(file.exists(paste(WorkPath, "/pairwiseCC.rda", sep=""))) "id_of_table" else NULL
  if (!GeneSetComparison) {
    GeneId1=CreateGeneTable(GeneTable=GeneId
                            , AllGenesInAnalysis=GeneId1B
                            , InsertAfterColumn=GeneIdColumnToMatch
                            , RunChromDistBarPlot=RunChromDistBarPlot
                            , RunGOOvE=DoGOOvE
                            , MinObserved=MinObserved
                            , MinRatio=MinRatio
                            , RedoExpectedTable=RedoExpectedTable
                            , FolderPath=paste(projectPath, "/Output/", AnalysisName, sep="")
                            , ReturnHtmlCode=TRUE
                            , GeneTableFile=""
                            , AnnotationsTableFile="Annotations.html"
                            , DoMasterAnnotations=DoMasterAnnotations
                            , ChromDistBarPlotFile=paste(AnalysisName, "-ChromDist.jpg")
                            , HeaderStringsVec=""
                            , TableCaption=""
                            , GeneIdColumnToMatch=GeneIdColumnToMatch
                            , TableID=TableID
                            , projectPath=projectPath)
    
    # Specifically designed for pairwise comparison.
    if (file.exists(paste(WorkPath, "/pairwiseCC.rda", sep=""))) {
      ncolgenetable <- ncol(GeneId)+1*GeneId1$DoAnnotations
      save(ncolgenetable, file=paste(WorkPath, "/ncolgenetable.rda", sep=""))
      # the final number of columns may be one more (annotation column) if annotation has been done successfully
    }
    
    OEhtml <- GeneId1$GOTableHtmlCode
    GeneId1 <- GeneId1$GeneTableHtmlCode
  } else {
    
    #6/8/07 generate separate gene table html files for gene set comparison analysis.
    GeneId1 <- NULL
    for (i in 1:length(GeneTablePartition)) {
      if (i==1) {
        startRow <- 1
        endRow <- GeneTablePartition[i]
      } else {
        startRow <- GeneTablePartition[i-1] + 1
        endRow <- GeneTablePartition[i]
      }
      GeneIdUnit <- GeneId[startRow:endRow,]
      GeneRowNumUnit <- GeneRowNum[startRow:endRow]
      GeneTableFile <- paste(GeneTableFileName, i, ".html", sep="")
      CreateGeneTable(GeneTable=GeneIdUnit
                      , AllGenesInAnalysis=NULL
                      , InsertAfterColumn=GeneIdColumnToMatch
                      , RunChromDistBarPlot=RunChromDistBarPlot
                      , RunGOOvE=DoGOOvE
                      , MinObserved=MinObserved
                      , MinRatio=MinRatio
                      , RedoExpectedTable=RedoExpectedTable
                      , FolderPath=paste(projectPath, "/Output/", AnalysisName, sep="")
                      , ReturnHtmlCode=FALSE
                      , GeneTableFile=GeneTableFile
                      , AnnotationsTableFile=""
                      , DoMasterAnnotations=FALSE
                      , ChromDistBarPlotFile=""
                      , HeaderStringsVec=""
                      , TableCaption=GeneTableCaption
                      , GeneIdColumnToMatch=GeneIdColumnToMatch
                      , GeneRowNum=GeneRowNumUnit
                      , projectPath=projectPath)
      
      GeneId1 <- c(GeneId1, ifelse(is.null(GeneId1), "", " ... "), 
                   paste("<a href=\"", GeneTableFile, "\">", 
                         "Gene Sets ", (i-1)*GeneSetUnit+1, " - ", ifelse(i<length(GeneTablePartition), i*GeneSetUnit, length(GeneSetIndex)), "</a>", sep=""))
    }
    OEhtml <- NULL
    
    ## Color code KEGG pathway
    #Generate URL to color code up/down regulated genes for KEGG pathway by using KEGG API  Qihao 8/15/2013
    ColorCodeUseKEGGAPI<-function(KEGGpathwayID,GeneIdentifersList,GeneExpressionToComputeFoldChange=NULL,Pvalues){
      #save(list=c('KEGGpathwayID','GeneIdentifersList','GeneExpressionToComputeFoldChange','Pvalues'),file='c:/temp/test')
      #Qihao modified codes 7/22/2015.
      if (!(is.null(GeneExpressionToComputeFoldChange))){
        GeneExpressionToComputeFoldChange<-sapply(GeneExpressionToComputeFoldChange,function(x){as.double(levels(x)[x])})
      }
      #Qihao added codes to handle genes without gene symbol.
      if (all((GeneIdentifersList==''))|all(is.na(GeneIdentifersList))){
        return('No graph available')
      }
      if(any((GeneIdentifersList==''|is.na(GeneIdentifersList)))){
        MisssymbolGenes<-which(((GeneIdentifersList!='')&(!(is.na(GeneIdentifersList)))))
        GeneIdentifersList<-GeneIdentifersList[MisssymbolGenes]
        if (!(is.null(GeneExpressionToComputeFoldChange))){GeneExpressionToComputeFoldChange<-GeneExpressionToComputeFoldChange[MisssymbolGenes,,drop=FALSE]}
        Pvalues<-Pvalues[MisssymbolGenes]
      }
      Pvalues<-sapply(as.character(levels(Pvalues)[Pvalues]),function(x){if(x=='< 1e-07'){as.numeric(x=0.0000000)}else{as.numeric(x)}})
      index<-(Pvalues<=0.01)
      GeneIdentifersList<-(GeneIdentifersList[index])
      
      if (sum(index)==0){
        #URLstring<-paste('<A href=\"http://www.kegg.jp/kegg-bin/show_pathway?map=',KEGGpathwayID,'\">','View','</A>',sep='')
        return('No graph available')
      }else{
        
        if (is.null(GeneExpressionToComputeFoldChange)){		
          URLstring<-paste(lapply(GeneIdentifersList,function(x){paste(x,'+%23D74B4B,',sep='')}),collapse='/')
          #GeneSetdescriptions <- paste("<A HREF=\"http://www.kegg.jp/kegg-bin/show_pathway?", GeneSetId, "\">", GeneSetdescriptions, "</A>", sep="")	
        }else{
          #Assume GeneExpressionToComputeFoldChange must be a matrix having only two columns.
          GeneExpressionToComputeFoldChange<-GeneExpressionToComputeFoldChange[index,,drop=FALSE]
          #Qihao modified codes. 7/22/2015
          UpDownRegulatedIndex<-apply(GeneExpressionToComputeFoldChange,1,function(x){if(x[1]==x[2]){c(1)}else{if(x[1]>x[2]){c(2)}else{c(3)}}})
          
          if (any(UpDownRegulatedIndex==2)){
            URLstring<-paste(lapply(GeneIdentifersList[(UpDownRegulatedIndex==2)],function(x){paste(x,'+%23D74B4B,',sep='')}),collapse='/')
          }
          if (any(UpDownRegulatedIndex==3)){
            DownURLstring<-paste(lapply(GeneIdentifersList[(UpDownRegulatedIndex==3)],function(x){paste(x,'+%233399CC,',sep='')}),collapse='/')
            if(exists('URLstring')){URLstring<-paste(URLstring,DownURLstring,sep='/')}else{URLstring<-DownURLstring}
          }
          if (any(UpDownRegulatedIndex==1)){
            EqualURLstring<-paste(URLstring,lapply(GeneIdentifersList[(UpDownRegulatedIndex==1)],function(x){paste(x,'+%23CDB99C,',sep='')}),collapse='/')
            if(exists('URLstring')){URLstring<-paste(URLstring,EqualURLstring,sep='/')}else{URLstring<-EqualURLstring}
          }
          #save(list=c('URLstringUp','URLstringDown','URLstringNormal','UpDownRegulatedIndex'),file='c:/temp/test')
          #URLstring<-paste(URLstringUp,URLstringDown,URLstringNormal,collapse='/')
        }
        
        URLstring<-paste('<A href=\"http://www.kegg.jp/kegg-bin/show_pathway?map=',KEGGpathwayID,'&multi_query=',URLstring,'\">','View','</A>',sep='')
        return(URLstring)
        
      }
    }
    #save(list=c('GeneId1','GeneSetTable','GeneId','GeneSetList','GeneSetId','GeneSetdescriptions','GeneIdNames'),file='c:/temp/qqk')
    # save.image(file='c:/temp/qqy') 
    if (AnalysisType == "PathwayComparison"){
      #Qihao fixed #515
      if((PathwayType == "Kegg")&&(!(paired))){
        KEGG.colorcode.pathwayName<-GeneSetDescriptionName
        j<-0
        KEGG.color.Position<-unlist(lapply(GeneSetList,length))
        
        if (length(unique(classlabels)) == 2){
          
          for (i in 1:length(GeneSetId)){
            #List<-GeneId[1+j:length(GeneSetList[[i]]),grep('*symbol',GeneIdNames,ignore.case=TRUE)]
            KEGG.color.Begin<-1+j
            KEGG.color.End<-j+KEGG.color.Position[i]
            #KEGG.colorcode.pathwayName[i]<-ColorCodeUseKEGGAPI(GeneSetId[i],GeneId[KEGG.color.Begin:KEGG.color.End,grepl('*symbol',colnames(GeneId),ignore.case=TRUE)],subset(GeneId[KEGG.color.Begin:KEGG.color.End,(grepl('Geom mean',colnames(GeneId)))]),GeneId[KEGG.color.Begin:KEGG.color.End,grepl('p-value',colnames(GeneId),ignore.case=TRUE)])
            KEGG.colorcode.pathwayName[i]<-ColorCodeUseKEGGAPI(GeneSetId[i],GeneId[KEGG.color.Begin:KEGG.color.End,grepl('*EntrezID',colnames(GeneId),ignore.case=TRUE)],subset(GeneId[KEGG.color.Begin:KEGG.color.End,(grepl('Geom mean',colnames(GeneId)))]),GeneId[KEGG.color.Begin:KEGG.color.End,grepl('p-value',colnames(GeneId),ignore.case=TRUE)]) ## Ting used EntrezID to generate KEGG pathway web links with map coloring 4/18/2017
            j<-KEGG.color.End
          }			
          
        }else{
          
          for (i in 1:length(GeneSetId)){
            #List<-GeneId[1+j:length(GeneSetList[[i]]),grep('*symbol',GeneIdNames,ignore.case=TRUE)]
            KEGG.color.Begin<-1+j
            KEGG.color.End<-j+KEGG.color.Position[i]
            #KEGG.colorcode.pathwayName[i]<-ColorCodeUseKEGGAPI(GeneSetId[i],GeneId[KEGG.color.Begin:KEGG.color.End,grepl('*symbol',colnames(GeneId),ignore.case=TRUE)],as.null(),GeneId[KEGG.color.Begin:KEGG.color.End,grepl('p-value',colnames(GeneId),ignore.case=TRUE)])
            KEGG.colorcode.pathwayName[i]<-ColorCodeUseKEGGAPI(GeneSetId[i],GeneId[KEGG.color.Begin:KEGG.color.End,grepl('*EntrezID',colnames(GeneId),ignore.case=TRUE)],as.null(),GeneId[KEGG.color.Begin:KEGG.color.End,grepl('p-value',colnames(GeneId),ignore.case=TRUE)]) ## Ting used EntrezID to generate KEGG pathway web links with map coloring 4/18/2017
            j<-KEGG.color.End
          }
          
        }
        if (!(is.null(GeneSetTable1))){
          GeneSetTable <- cbind(GeneSetId, GOOntologySig, GeneSetdescriptions,KEGG.colorcode.pathwayName,NgenesGeneSet, if (HasClasses) HeatmapGeneSet,
                                Trunc(pGeneSet$pLS, cutoff=alpha, TruncLevel=10^(-5)),
                                if(DoKStest) Trunc(pGeneSet$pKS, cutoff=alpha, TruncLevel=10^(-5)),
                                if(!GeneSetGSA) NULL else Trunc(pGeneSet$pGSA, cutoff=alpha, npermGSA=npermGSA, GSAscore=GSAscore, classlabels=classlabels),                      
                                if(!Hotelling) NULL else Trunc(pGeneSet$pHotelling, cutoff=alpha),
                                if(!GeneSetGlobalTest) NULL else Trunc(pGeneSet$pGlobal, cutoff=alpha))
          GeneSetTableNames <- c(GeneSetIdName, GOOntologyName, GeneSetDescriptionName
                                 ,'Color Coded KEGG Pathway Graphs'
                                 ,"Number of <NOBR>genes</NOBR>", if (HasClasses) "Heatmap link",
                                 "LS permutation <NOBR>p-value</NOBR>", 
                                 if(DoKStest) "KS permutation <NOBR>p-value</NOBR>",
                                 if(!GeneSetGSA) NULL else "Efron-Tibshirani's GSA test <NOBR>p-value</NORB>",
                                 if(!Hotelling) NULL else "Hotelling test <NOBR>p-value</NOBR>",
                                 if(!GeneSetGlobalTest) NULL else "Goeman's global test <NOBR>p-value</NOBR>")
          dimnames(GeneSetTable)[[2]] <- GeneSetTableNames
          GeneSetTable1 <- CreateHtmlTable(GeneSetTable)
        }
      }
    }
  }
  
  
  ## For greedy pair method in class prediction we output gene table sorted by gene pairs             ##
  if (AnalysisType == "ClassPrediction" && !is.null(genepairIndex)) {
    tmp <- GeneId[match(genepairIndex, index[ordercode]),,drop = FALSE]
    tmp <- cbind(data.frame(Pair = rep(1:(nrow(tmp)/2), each =2)), tmp)
    GeneIdPair=CreateGeneTable(GeneTable=tmp
                               , InsertAfterColumn=GeneIdColumnToMatch+1
                               , RunChromDistBarPlot=FALSE
                               , RunGOOvE=FALSE
                               , MinObserved=MinObserved
                               , MinRatio=MinRatio
                               , RedoExpectedTable=RedoExpectedTable
                               , FolderPath=paste(projectPath, "/Output/", AnalysisName, sep="")
                               , ReturnHtmlCode=TRUE
                               , GeneTableFile=""
                               , AnnotationsTableFile="Annotations.html"
                               , DoMasterAnnotations=DoMasterAnnotations
                               , ChromDistBarPlotFile=""
                               , HeaderStringsVec=""
                               , TableCaption=""
                               , GeneIdColumnToMatch=GeneIdColumnToMatch+1
                               , projectPath=projectPath)$GeneTableHtmlCode
  } else GeneIdPair <- NULL
  
  ## FOR QUANTITATIVE TRAITS OR ANY PAIRED ANALYSIS, RE-SORT GENES AND CREATE A SECOND TABLE OF SIGNIFICANT GENES.
  
  if (paired) {
    if(length(meandiff) > 1) {
      GeneId <- GeneId[rev(order(meandiff)), ,drop=FALSE]
    }
  }
  
  if (AnalysisType == "Correlation") {
    if(length(classifiers$correlation) > 1) {
      GeneId <- GeneId[rev(order(classifiers$correlation)), ,drop=FALSE]
    }
  }
  
  if ((paired)|(AnalysisType == "Correlation")) {
    GeneId2=CreateGeneTable(GeneTable=GeneId
                            , AllGenesInAnalysis=GeneId1B
                            , InsertAfterColumn=GeneIdColumnToMatch
                            , RunChromDistBarPlot=TRUE
                            , RunGOOvE=DoGOOvE
                            , MinObserved=MinObserved
                            , MinRatio=MinRatio
                            , RedoExpectedTable=RedoExpectedTable
                            , FolderPath=paste(projectPath, "/Output/", AnalysisName, sep="")
                            , ReturnHtmlCode=TRUE
                            , GeneTableFile=""
                            , AnnotationsTableFile="Annotations.html"
                            , DoMasterAnnotations=DoMasterAnnotations
                            , ChromDistBarPlotFile=paste(AnalysisName, " -ChromDist.jpg")
                            , HeaderStringsVec=""
                            , TableCaption=""
                            , GeneIdColumnToMatch=GeneIdColumnToMatch
                            , projectPath=projectPath
    )
    GeneId2 <- GeneId2$GeneTableHtmlCode
  } else {
    GeneId2 <- NULL
  }
  
  ## RETURN STRINGS CONTAINING HTML CODE TO BE INSERTED INTO HTML OUTPUT FILES, AND OTHER PARAMETERS.
  out <- list(GeneSetTable1=GeneSetTable1, GeneSetTable2=GeneSetTable2,
              GeneId1=GeneId1, GeneId2=GeneId2, OEhtml=OEhtml, GeneIdPair=GeneIdPair,
              NumGenesInAnalysis=NumGenesInAnalysis, ProbeSets=ProbeSets
              , GeneIndex=index # return significant gene index, Yong 8/23/06
              , GONumTotCat=GONumTotCat, GONumSigCat=GONumSigCat # return number of total investigated and significant GO categories, Yong 4/2/07
              ,GeneIdForIngenuity=GeneId #Qihao modified this.
  )
  invisible(out)      
}
