# R script for creating volcano plot (nclass=2) or parallel coordinate plot (nclass>2).
# Modify to use sendplot to create a dynamic plot. MLI 11/16/2010			 
PCP <- function(WorkPath,projectPath,AnalysisName,RunLFDR,paired,ClassVariableName,ind.deg.lfdr) {
  if(!LoadRPackage("sendplot")) stop("Could not download the R package, sendplot.")
  #library(PKGNAME, character.only=TRUE)
  options(warn=-1)
  setwd(WorkPath)
  params <- scan("class_perm_params.txt", quiet = TRUE)
  codeReg <- params[14]
  HasBlock <- params[17]
  codeFoldchange <- params[18]
  Foldthresh <- params[19]  # MLI added 2/13/2013
  if (Foldthresh <= 1) codeFoldchange <- 0
  codeT <- params[4]
  AlphaT <- params[5]
  yhorizon <- -log10(AlphaT)
  classsize <- scan("LR_data.txt", skip=1, nlines=1, quiet = TRUE)
  load("experiments.rda") # experiments
  cls <- experiments[,2]
  if (HasBlock == 1) {
    blockcls <- experiments[,3]
    cls <- paste(cls, blockcls, sep=", ")
  } 
  ncls <- length(unique(cls)) 
  load("GeneIdNames.rda")
  load("GeneId1.rda")        # GeneId1 contains only filtered genes.
  if (! "Symbol" %in% GeneIdNames) {
    #GeneId1Folder <- file.path(projectPath, "Annotations", 
    #                          paste(GeneIdNames[1], ".txt", sep=""))
    #if (file.exists(file.path(projectPath, "Annotations", "Symbol.txt")) & 
    #    file.exists(GeneId1Folder)) {
    #  GeneId1total <- read.table(GeneId1Folder, sep="\t", header=TRUE, fill=TRUE, as.is=TRUE, 
    #                             comment.char="", quote="")[,2]
    #  Row <- match(GeneId1, GeneId1total)
    #  Symbol <- read.table(file.path(projectPath, "Annotations", "Symbol.txt"),  
    #                       sep="\t", header=TRUE, fill=TRUE, as.is=TRUE, 
    #                       comment.char="", quote="")[,2]        
    #  genelabelfilter <- Symbol[Row] # filtered genes only
    #  includesymbol <- TRUE
    #} else {
      includesymbol <- FALSE
    #}
  } else {
    load(paste("GeneId", which(GeneIdNames == "Symbol")[1], ".rda", sep=""))
    genelabelfilter <- get(paste("GeneId", which(GeneIdNames == "Symbol")[1], sep=""))
    includesymbol <- TRUE
  }

  nsiggenes <- length(readLines("DEGs.txt")) - 2
  if (nsiggenes > 0) {
    if (ncls == 2) {
      # volcano plot
      cat("Preparing the Volcano plot ...", "", sep="\n")
      pvalues <- scan("Pvalues.txt", quiet = TRUE)
      pvalues[pvalues == 0] <- 1.0e-7 # Follow Fortran threshold. 1/19/2011
      nlog10p <- -log10(pvalues)  		  
      # Modify to include local fdr
      indexg <- if (!RunLFDR)  {
        read.table("DEGs.txt", skip=2, header=FALSE)[,1]
      } else {
        ind.deg.lfdr # defined in 'GetComparisonResults.R'
      }		  
      if (paired==FALSE) {
        load("meanLR1.rda")
        meanLR1 <- meanLR
        load("meanLR2.rda")
        meanLR2 <- meanLR
        meanLRdiff <- meanLR1 - meanLR2
      } else {
        load("meandiff.rda")
        meanLRdiff <- meandiff
      }    ## Lori added this for paired condition 03/30/09
      xlim <- range(meanLRdiff, na.rm=TRUE)
      ylim <- range(nlog10p, na.rm=TRUE)
      x.pos <- meanLRdiff
      y.pos <- nlog10p
      
      x.pos_ClassComparisonTmp<<-x.pos
      y.pos_ClassComparisonTmp<<-y.pos
      ClassVariableName_ClassComparisonTmp<<-ClassVariableName
      indexg_ClassComparisonTmp<<-indexg
      xlim_ClassComparisonTmp<<-xlim
      ylim_ClassComparisonTmp<<-ylim
      codeT_ClassComparisonTmp<<-codeT
      RunLFDR_ClassComparisonTmp<<-RunLFDR
      Foldthresh_ClassComparisonTmp<<-Foldthresh
      codeFoldchange_ClassComparisonTmp<<-codeFoldchange
      yhorizon_ClassComparisonTmp<<-yhorizon
      meanLRdiff_ClassComparisonTmp<<-meanLRdiff
      nlog10p_ClassComparisonTmp<<-nlog10p
      
      plot.call=c("plot(x.pos_ClassComparisonTmp,y.pos_ClassComparisonTmp,xlab='Log2(Fold change)', ylab='-Log10(p-values)',
                  main=ClassVariableName_ClassComparisonTmp)")
      plot.extras <- paste0("par(new=TRUE);",
                            "plot(meanLRdiff_ClassComparisonTmp[indexg_ClassComparisonTmp], nlog10p_ClassComparisonTmp[indexg_ClassComparisonTmp], pch=20, col='blue',xlim=xlim_ClassComparisonTmp, ylim=ylim_ClassComparisonTmp, xlab='', ylab='');",
                            "if (codeT_ClassComparisonTmp == 1 & !RunLFDR_ClassComparisonTmp) abline(h=yhorizon_ClassComparisonTmp,lty=3);",
                            "if (codeT_ClassComparisonTmp == 1 & codeFoldchange_ClassComparisonTmp == 1 & !RunLFDR_ClassComparisonTmp) {abline(v=log2(Foldthresh_ClassComparisonTmp),lty=3);abline(v=-log2(Foldthresh_ClassComparisonTmp),lty=3)}")
      
      xy.labels = data.frame(ProbeSet = as.character(GeneId1))
      colnames(xy.labels) <- GeneIdNames[1]
      if (includesymbol) xy.labels <- cbind(xy.labels, data.frame(GeneSymbol = as.character(genelabelfilter)))
      direct <- paste(projectPath, "/Output/", AnalysisName, "/", sep="")
      pltfilename <- "VolcanoPlot"
      xy.send(plot.call=plot.call,
              y.pos=y.pos,x.pos=x.pos,
              xy.labels = xy.labels,
              plot.extras=plot.extras,	
              image.size="800x600",
              fname.root= pltfilename,
              dir=direct,
              font.size=18)			 
      
      rm(x.pos_ClassComparisonTmp,y.pos_ClassComparisonTmp,
         ClassVariableName_ClassComparisonTmp,indexg_ClassComparisonTmp,
         xlim_ClassComparisonTmp,ylim_ClassComparisonTmp,codeT_ClassComparisonTmp,
         RunLFDR_ClassComparisonTmp,Foldthresh_ClassComparisonTmp,
         codeFoldchange_ClassComparisonTmp,yhorizon_ClassComparisonTmp,
         meanLRdiff_ClassComparisonTmp,nlog10p_ClassComparisonTmp, envir = .GlobalEnv)
      gc()
    } else  {
      # PCP plot for more than 2 classes
      cat("Preparing the PCP plot ...", "", sep="\n")
      indexg <- read.table("DEGs.txt", skip=2, header=FALSE)[,1]
      lr <- read.table("LR_data.txt", skip=2+1*HasBlock, header=FALSE)
      lr <- as.matrix(lr)
      lr[lr == -9999999] <- NA
      if (codeReg == 1) {
        FILTER <- lr[, ncol(lr)]
        lr <- lr[FILTER == 1, -ncol(lr)]
      }	
      lr2 <- aggregate(t(lr[indexg,, drop=FALSE]), list(cls), mean, na.rm=TRUE)
      # lr2: matrix of mean values for each group
      lr2 <- t(lr2) # rows =genes, col=class
      x <- t(matrix(as.numeric(lr2[-1, ]), nc=ncls) )
      rownames(x) <- cls.unique <- lr2[1,]
      y <- 1:ncls
      
      x_ClassComparisonTmp<<-x
      y_ClassComparisonTmp<<-y
      HasBlock_ClassComparisonTmp<<-HasBlock
      ncls_ClassComparisonTmp<<-ncls
      cls.unique_ClassComparisonTmp<<-cls.unique
      
      plot.call= "par(mar=c(5,4,2,10)+.1); matplot(x_ClassComparisonTmp, y_ClassComparisonTmp, type='l', axes=FALSE, xlab='Mean of log ratios/intensities',
      ylab=ifelse(HasBlock_ClassComparisonTmp, 'Class, Block', 'Class'))"
      plot.extras <- "box(); axis(4, at=1:ncls_ClassComparisonTmp, labels=cls.unique_ClassComparisonTmp, las=2); axis(1); cex <- .8; nchars <- 5"
      x.pos <- as.numeric(x)
      y.pos <- rep(y, ncol(x))
      xy.labels = data.frame(ProbeSet = rep(as.character(GeneId1[indexg]), each=ncls))
      colnames(xy.labels) <- GeneIdNames[1]
      if (includesymbol) xy.labels <- cbind(xy.labels, 
                                            data.frame(GeneSymbol = rep(genelabelfilter[indexg], each=ncls)))
      direct <- paste(projectPath, "/Output/", AnalysisName, "/", sep="")
      pltfilename <- "PCPlot"
      xy.send(plot.call=plot.call,
              y.pos=y.pos,x.pos=x.pos,
              xy.labels = xy.labels,
              plot.extras=plot.extras,	
              image.size="800x600",
              fname.root=pltfilename,
              dir=direct,
              font.size=18)
      
      rm(x_ClassComparisonTmp,y_ClassComparisonTmp,HasBlock_ClassComparisonTmp,
         ncls_ClassComparisonTmp,cls.unique_ClassComparisonTmp, envir = .GlobalEnv)
      gc()
    } # End of if (ncls == 2)
    # Modify the HTML file manually to show the hint and change plot title
    pltfilenamefull <- paste(direct, pltfilename, ".html", sep="")
    pltfile <- scan(pltfilenamefull, "", sep="\n", quiet = TRUE)	
    ind1 <- grep("div class", pltfile)
    pltfile <- c(pltfile[1:(ind1-1)], 
                 "<P>Hover the mouse cursor over a point to view gene information. <BR>PS. Internet Explorer users need to enable ActiveX control for this to work. 
					      Go back and click the HELP button for more detail about the instruction.</P>",
                 pltfile[ind1:length(pltfile)])
    ind2 <- grep("<title>", pltfile)			
    pltfile <- c(pltfile[1:(ind2-1)],
                 paste("    <title>", ifelse(ncls==2, "Volcano plot", "Parallel coordinate plot"), "</title>", sep=""),
                 pltfile[(ind2+1):length(pltfile)])				
    cat(pltfile, file = pltfilenamefull, sep="\n")
    unlink(paste(direct, "*.tif", sep = "")) # remove unnecessary files
  }
} # End of if (require(sendplot))
