# Purpose: create <RunFortran.bat>
createfortran.classComparison <- function(ArrayToolsPath, outputpath) {
  
  # replace one forward slash with one backward slash for running in COMMAND PROMPT
  outputpath2 <- gsub("/", "\\\\", outputpath)
  ArrayToolsPath <- normalizePath(ArrayToolsPath)
  out <- c("@echo off",
           substring(ArrayToolsPath, 1, 2),
           paste0("cd \"", ArrayToolsPath, "\\Fortran\""),
           paste0("ClassComparison.exe^\n \"",
                  outputpath2, "\"^\n 1> \"",
                  outputpath2, "\\FortranOutput.txt\"^\n 2> \"",
                  outputpath2, "\\FortranError.txt\""))
  cat(out, file = file.path(outputpath, "RunFortran.bat"), sep = "\n")
}

# function to compute number of all possible permutaitons
# From Misc.R.
ComputNumberOfPermutations <- function(pheno,paired=FALSE,surv=FALSE) {
  
  if(paired){
    nArrays <- length(pheno)/2
    if(nArrays > 17.5) res <- 40000
    if(nArrays < 17.5) res <- 2^(nArrays-1)
  }
  if(surv) {
    nArrays <- length(pheno)
    if(nArrays < 9) {
      res <- 1
      for(i in 1:nArrays) { res <- res*i }
      res <- res/2
    } else { res <- 40000 }
  }
  if(!paired & !surv) {
    nArrays <- length(pheno)
    groups <- unique(pheno)
    ngroups <- length(groups)
    n <- double(ngroups)
    for(i in 1:ngroups) n[i] <- length(pheno[pheno == groups[i] ] )
    if(sum(n) != nArrays ) print("ERRROR in R function ComputNumberOfPermutations")
    num <- 1:nArrays
    denom <- NULL
    for (i in 1:ngroups) denom <- c(denom, 1:n[i]) 
    res <- 1
    for(i in 1:nArrays) res <- res* num[i]/denom[i]
    maxNum <- max(n)
    for( i in 1:maxNum) {
      k=length(n[n==i])
      if(k > 1) {
        for(j in 2:k) {res <- res/j }
      }
    }
  }
  if(res > 40000) res <- 40000
  return(res)
}

# From Misc.R.
ComputNumberOfPermutationsBlock <- function(block) {
  min(prod(gamma(table(block)+1)), 10000)
}

#CREATE FUNCTION FOR WRITING NETAFFX LINKS.
CreateNetAffxBatchLinks <- function(FirstIndex, LastIndex, LastLink=FALSE) {
  #Put comma after all except last ProbeSet in the chunk.
  if (FirstIndex < LastIndex) {
    ProbeSetChunk <- c(paste(ProbeSets[FirstIndex:(LastIndex-1)], rep(",", LastIndex-FirstIndex), sep=""), ProbeSets[LastIndex])
    #cat("<A HREF=\"https://www.affymetrix.com/LinkServlet?probeset="
    #	, ProbeSetChunk
    #	, paste("\" TARGET=_blank>Batch query NetAffx for ProbeSets #", FirstIndex, "-", LastIndex, "listed below</A>")
    #	, ifelse(LastLink,"","<BR>")
    #	, file=ExternalFileName, sep="\n", append=TRUE)
    cat(paste("Batch query NetAffx for ProbeSets #", FirstIndex, "-", LastIndex, "listed below")
        , ifelse(LastLink,"","<BR>")
        , file=ExternalFileName, sep="\n", append=TRUE)
  } else {
    #	cat("<A HREF=\"https://www.affymetrix.com/LinkServlet?probeset="
    #		, ProbeSets[LastIndex]
    #		, paste("\" TARGET=_blank>Batch query NetAffx for ProbeSets #", FirstIndex, "-", LastIndex, "listed below</A>")
    #		, ifelse(LastLink,"","<BR>")
    #		, file=ExternalFileName, sep="\n", append=TRUE)
    cat(paste("Batch query NetAffx for ProbeSets #", FirstIndex, "-", LastIndex, "listed below")
        , ifelse(LastLink,"","<BR>")
        , file=ExternalFileName, sep="\n", append=TRUE)
  }
}

RemoveInvalidChar <- function(x) {
  # Purpose:
  #   Remove special characters which were not valid in windows filename
  # Input:
  #   x: a character string
  # Output:
  #   a character string
  # Example:
  #   RemoveInvalidChar(c("A\\B", "A\"B", "A/B", "A*B", "A[B", "AB]", "A|B", "A:B", "A<B", "A>B", "A?B"))
  tmp <- gsub("\\\\", "", x)
  tmp <- gsub("\"", "", tmp)
  tmp <- gsub("/", "", tmp, fixed=TRUE)
  tmp <- gsub("*", "", tmp, fixed=TRUE)
  tmp <- gsub("[", "", tmp, fixed=TRUE)
  tmp <- gsub("]", "", tmp, fixed=TRUE)
  tmp <- gsub("|", "", tmp, fixed=TRUE)
  tmp <- gsub(":", "", tmp, fixed=TRUE)
  tmp <- gsub("<", "", tmp, fixed=TRUE)
  tmp <- gsub(">", "", tmp, fixed=TRUE)
  tmp <- gsub("?", "", tmp, fixed=TRUE)
  tmp
}

##                                                                                    ##
## FUNCTION:  CreatePath                                                              ##
##                                                                                    ##
## THIS FUNCTION CONSTRUCTS THE FOLDERS FOR A GIVEN PATH,                             ##
## IF THE GIVEN PATH DOES NOT YET EXIST.                                              ##
##                                                                                    ##
CreatePath <- function(FolderPath) {
  ## Create FolderPath if it doesn't yet exist.
  Folders <- unlist(strsplit(FolderPath, split="/"))
  Folders <- unlist(strsplit(Folders, split="\\\\"))
  FolderPath <- Folders[1]
  if (length(Folders) > 1) {
    for (i in 2:length(Folders)) {
      FolderPath <- paste(FolderPath, Folders[i], sep="/")
      if (file.access(FolderPath)==-1) dir.create(FolderPath)
    }
  }
}

outputClassname <- function(classname, classsize) { # a helper function, Yong 12/15/06
  str1 <- ""
  bclasssize <- !missing(classsize)
  for (i in 1:length(classname)) {
    str2 <- paste("<font size=2>Class ", i, ": ", "<I>", classname[i], "</I>", 
                  if (bclasssize) paste0(" (", classsize[i], " samples)"), "</font>", sep="") # add classsize, MLI 5/16/17
    str1 <- paste(str1, str2, sep="")
    if (i < length(classname)) str1 <- paste(str1, "; ", sep="")
  }
  str1 <- paste(str1, ".", sep="")
}

# added codes to output a file for Ingenuity Pathway Analysis. 7/15/2013
CreateIPAOutput<-function(IngenuityOutput,GeneIdForIngenuity,ReferenceClass,UseCorrelationAsFoldChange,classlabels){
  Output<-matrix(NA,ncol=1,nrow=nrow(GeneIdForIngenuity))
  
  #Use Gene Symbol as the Gene Identifer.
  #Qihao added codes to handle the case that project gene identifer worksheet doesn't have gene symbol. 2014/2/26
  if (any(grepl('symbol',colnames(GeneIdForIngenuity),ignore.case = TRUE))){
    Output[,1]<-as.character(GeneIdForIngenuity[,(grep('symbol',colnames(GeneIdForIngenuity),ignore.case = TRUE)[1])])
    
    #Check whether the inputted GeneID object contains "NA" instead of blank elements. 2014/3/5
    Output[is.na(Output),1]<-''
    #Use EntrezID as second order gene identifer
    Missindex<-(Output[,1]=='')
    if (any(Missindex)){
      #Qihao added codes to check whether the project annotation worksheet has EntrezID or not. 1/15/2014
      if (any(grepl('EntrezID',colnames(GeneIdForIngenuity),ignore.case = TRUE))){
        Output[Missindex,1]<-as.character(GeneIdForIngenuity[Missindex,(grep('EntrezID',colnames(GeneIdForIngenuity),ignore.case = TRUE)[1])])
      }
    }
    Output[is.na(Output),1]<-''
    Missindex<-(Output[,1]=='')
    if (any(Missindex)){
      if (any(grepl('UniqueID',colnames(GeneIdForIngenuity),ignore.case = TRUE))){
        Output[Missindex,1]<-as.character(GeneIdForIngenuity[Missindex,grep('UniqueID',colnames(GeneIdForIngenuity),ignore.case = TRUE)])
      }else{
        Output[Missindex,1]<-as.character(GeneIdForIngenuity[Missindex,grep('ProbeSet',colnames(GeneIdForIngenuity),ignore.case = TRUE)])
      }
    }
  }else{
    if (any(grepl('UniqueID',colnames(GeneIdForIngenuity),ignore.case = TRUE))){
      Output[,1]<-as.character(GeneIdForIngenuity[,grep('UniqueID',colnames(GeneIdForIngenuity),ignore.case = TRUE)])
    }else{
      Output[,1]<-as.character(GeneIdForIngenuity[,grep('ProbeSet',colnames(GeneIdForIngenuity),ignore.case = TRUE)])
    }
  }
  #Compute fold change
  ComputeFC<-function(index1,index2,Data=GeneIdForIngenuity){
    #look for the Geom mean column.
    GeoMindex<-grep("Geom mean*", names(Data),ignore.case = TRUE)
    ArrayOneIndex<-GeoMindex[index1]
    ArrayTwoIndex<-GeoMindex[index2]
    CFC<-(as.numeric(levels(Data[,ArrayOneIndex])[Data[,ArrayOneIndex]])/as.numeric(levels(Data[,ArrayTwoIndex])[Data[,ArrayTwoIndex]]))
    #Qihao modfied codes to handle NA Inf -Inf and NaN values. 6/12/2014 
    CFC<-as.matrix(sapply(CFC,function(x){if (is.finite(x)){if((x)<1){-x^(-1)}else{x}}else{x}}),ncol=1)
    colnames(CFC)<-paste('Fold Change of ', classlabels[index1],' VS ',classlabels[index2],sep='')
    return(round(CFC,digits=3))
  }
  if (UseCorrelationAsFoldChange){
    #Use correlation as fold change since Ingenuity only take the direction of the inputted fold change value. The magnitude of inputted fold change values would not be used in IPA. 
    IngenuityCorrelation<-as.numeric(levels(GeneIdForIngenuity[,grep('correlation',colnames(GeneIdForIngenuity),ignore.case = TRUE)])[GeneIdForIngenuity[,grep('correlation',colnames(GeneIdForIngenuity),ignore.case = TRUE)]])
    Output<-cbind(Output,IngenuityCorrelation)
    colnames(Output)[2]<-'Correlation coefficient'
  }else{
    if (ReferenceClass!='Automatically create the combination output'){
      ReferenceClassOrder<-which(classlabels==ReferenceClass)
      for (i in 1:(length(classlabels))){
        if (i != ReferenceClassOrder){
          Output<-cbind(Output,ComputeFC(i,ReferenceClassOrder))
        }
      }
    }else{
      #We assume that there are less than four classes in this data.
      ComOrder<-combn(length(classlabels),2)
      for (i in 1:ncol(ComOrder)){
        classlabels[ComOrder[1,i]]
        Output<-cbind(Output,ComputeFC(ComOrder[1,i],ComOrder[2,i]))
      }
    }
  }
  #Append P-value to the output
  #Qihao modified the code to output only Parametric p-value into the Ingenuity output text file when Permutation p-value column is given in the output HTML table.
  Output<-cbind(Output,sapply(as.character(GeneIdForIngenuity[,grep('p*value',colnames(GeneIdForIngenuity),ignore.case = TRUE)[1]]),function(x){if(x=='< 1e-07'){as.numeric(x=0.0000000)}else{as.numeric(x)}}))
  
  colnames(Output)[(ncol(Output))]<-gsub('<*.NOBR>','',(grep('p*value',colnames(GeneIdForIngenuity),ignore.case = TRUE,value=TRUE)[1]))
  colnames(Output)[1]<-c('Gene Identifier')
  write.table(Output,file=IngenuityOutput,quote=FALSE,row.names = FALSE,col.names=TRUE,sep='\t')
}


# Draw Chromsome distribution bar plot
ChromDistBarPlot <- function(GeneIds, projectPath, AnalysisName, ChromDistFileName="", AllGenesInAnalysis=NA) {
  ## GeneIds is the vector of gene ids which are to be searched against
  ## full genelist in first column of the Annotations.txt file.
  
  PrimaryIdFile <- paste(projectPath, "/Annotations/PrimaryId.txt", sep="")
  if (file.access(PrimaryIdFile) < 0) return(NULL)
  
  PrimaryId <- scan(PrimaryIdFile, what="", sep="\n", quiet = TRUE)
  PrimaryIdFile <- paste(projectPath, "/Annotations/", PrimaryId, ".txt", sep="")
  ChromosomeFile <- paste(projectPath, "/Annotations/Chromosome.txt", sep="")
  
  if (ChromDistFileName=="") {    # Amy modified 4/16/04.
    Outfile <- paste(projectPath, "/Output/", AnalysisName, "/", AnalysisName, " -ChromDist.jpg", sep="")
  } else {
    Outfile <- ChromDistFileName
  }
  
  if (file.access(PrimaryIdFile) + file.access(ChromosomeFile) < 0) return(NULL)
  
  PrimaryId <- read.table(file=PrimaryIdFile, sep="\t", header=TRUE, fill=TRUE, as.is=TRUE, comment.char = "", quote="")
  x <- read.table(file=ChromosomeFile, sep="\t", header=TRUE, fill=TRUE, as.is=TRUE, comment.char = "", quote="")
  x <- data.frame(PrimaryId[,2], x[,2])
  names(x) <- c("GeneId", "Chromosome")
  
  # Amy modified 7/4/06.
  if (is.null(AllGenesInAnalysis)) {
    PrimaryId <- read.table(file=PrimaryIdFile, sep="\t", header=TRUE, fill=TRUE, as.is=TRUE, comment.char = "", quote="")
    PrimaryId <- PrimaryId[,2]
    GenesInAnalysis <- PrimaryId
  } else {
    if ((length(AllGenesInAnalysis)==1) && (is.na(AllGenesInAnalysis))) {
      load(paste(WorkPath,"/GeneId1.rda",sep="")) #use load/save method, Yong 12/6/06. Use WorkPath 12/15/06
      GenesInAnalysis<- GeneId1
    } else {
      GenesInAnalysis<- AllGenesInAnalysis
    }
  }
  
  x<-x[match(GenesInAnalysis,PrimaryId[,2]),]  
  
  x1 <- x[match(GeneIds,x$GeneId),]
  
  xtable <- matrix(0,nrow=2,ncol=24)
  dimnames(xtable) <- list(NULL, c(as.character(1:22),"YX","?"))
  i <- 1
  for (chromosome in as.character(1:22)) {
    xtable[1,i] <- sum(x$Chromosome==chromosome)
    xtable[2,i] <- sum(x1$Chromosome==chromosome)
    i <- i+1
  }
  xtable[1,i] <- sum(x$Chromosome=="X"|x$Chromosome=="Y") #Lori 4/15/16
  xtable[2,i] <- sum(x1$Chromosome=="X"|x1$Chromosome=="Y") #Lori 4/15/16
  i <- i+1
  xtable[1,i] <- sum(x$Chromosome=="")
  xtable[2,i] <- sum(x1$Chromosome=="")
  
  ## Use the following if dropping missing values from analysis.
  xtable[1,24] <- xtable[1,24]/sum(xtable[1,])*100
  xtable[2,24] <- xtable[2,24]/sum(xtable[2,])*100
  rowsums <- apply(xtable[,1:23], 1, sum,na.rm=TRUE)
  if (rowsums[1] > 0) {
    xtable[1,1:23] <- xtable[1,1:23]/rowsums[1]*100
  } else {
    return(NULL)   #Exit this function.
  }
  if (rowsums[2] > 0) {
    xtable[2,1:23] <- xtable[2,1:23]/rowsums[2]*100
  } else {
    return(NULL)   #Exit this function.
  }
  
  if (xtable[1,24] > 50) {  #Amy added 11/18/04.
    cat("", paste("NOTE: The chromosomal distribution barplot will not be produced because ", round(xtable[1,24]), "% of the entire set of ", nrow(x), " genes had unknown chromosomal location.", sep=""), "", sep="\n")
    return(NULL)   #Exit this function.
  }
  
  if (xtable[2,24] > 50) {  #Amy added 11/18/04.
    cat("", paste("NOTE: The chromosomal distribution barplot will not be produced because ", round(xtable[2,24]), "% of the selected set of ", nrow(x1), " genes had unknown chromosomal location.", sep=""), "", sep="\n")
    return(NULL)   #Exit this function.
  }
  
  jpeg(Outfile, height=640, width=800)
  par(oma=c(1,1,7,1))
  # MLI modified to move the legend outside the barplot. 5/25/2007
  #  par(mar=c(7,4,4,2)+.1)
  #  par(xpd=TRUE)
  ylim <- range(xtable[, 1:23])
  bp <- barplot(xtable[,1:23], beside=TRUE, col=rep(c(2,4),24),    
                xlab="Chromosomal location", ylab="Percent of genes"
                #    , cex=.5
                #    , ylim=c(0,150)
                #    , axes=FALSE
                , cex.axis=.5
                , cex.names=.5, ylim = ylim
  )  #Use cex in R1.2.0.  Use cex.axis and cex.names in R1.6.1.  ??
  #  legend(x=25, y=130, legend=c("Entire geneset","Selected subset"))
  mtext("Compare chromosomal distribution\n", side=3, line=6.5, cex=1.2)
  mtext(paste("NOTE: ", round(xtable[1,24]), "% of the entire set of ", nrow(x), " genes and", sep=""), side=3, line=5, cex=1)
  mtext(paste(round(xtable[2,24]), "% of the selected set of ", nrow(x1), " genes had", sep=""), side=3, line=4, cex=1)
  mtext("unknown chromosomal locations and", side=3, line=3, cex=1)
  mtext("have been dropped from this analysis.", side=3, line=2, cex=1)
  #  if (sum(xtable[,24]) > 0) {
  #    mtext("Note: Bars may not add up to 100% of genes,", side=3, line=5, cex=1)
  #    mtext(paste("since ", round(xtable[1,24]), "% of the entire set of ", nrow(x), " genes and", sep=""), side=3, line=4, cex=1)
  #    mtext(paste(round(xtable[2,24]), "% of the selected set of ", nrow(x1), " genes had", sep=""), side=3, line=3, cex=1)
  #    mtext("unknown chromosomal locations.", side=3, line=2, cex=1)
  #  }
  mtext("Red=entire geneset, blue=selected subset.", side=3, line=1, cex=1)
  #  legend.text <- c("Entire geneset","Selected subset")
  #  legend(mean(range(bp)), ylim[2]*(-.3), legend.text, xjust = 0.5,
  #        fill=palette()[c(2,4)], horiz = TRUE) 
  dev.off()
}

SetRPackageDir <- function() {
  # Purpose:
  # Search for writable folder for installing R packages, especially for Windows Vista and up.
  # MLI 2/9/2011. 
  lib <- .libPaths()[1L]
  ok <- file.info(lib)$isdir & (file.access(lib, 2) == 0)
  if (length(lib) == 1L) {
    ok <- file.info(lib)$isdir %in% TRUE
    if (ok) {
      fn <- file.path(lib, paste("_test_dir", Sys.getpid(), sep="_"))
      unlink(fn, recursive = TRUE)
      res <- try(dir.create(fn, showWarnings = FALSE))
      if (inherits(res, "try-error") || !res) {
        ok <- FALSE
      } else unlink(fn, recursive = TRUE)
    } 
  }
  if (length(lib) == 1L && !ok) {
    userdir <- unlist(strsplit(Sys.getenv("R_LIBS_USER"), .Platform$path.sep))[1L]
    if (!dir.create(userdir, recursive = TRUE)) return(FALSE) else {
      .libPaths(userdir)
      return(TRUE)
    }
  }
  return(TRUE)
}


LoadRPackage <- function(pkgName, isBioconductor=FALSE, repos=NULL, type="win.binary", suppressUpdates=TRUE, bioctype=getOption("pkgType")) {
  # repos parameter was inserted to allow a different repository choice; eg r-forge. Ming-chung 3/4/2011
  #because warn set be 2 in CreateGeneTable.R which turn warning into error, it causes unnecessary trouble here. 
  if (!SetRPackageDir()) {
    require(tcltk)
    tkmessageBox(message="Writable R package directory cannot be found.",icon="error",type="ok")
    return(FALSE)
  }
  curwarn <- options()$warn
  options(warn=0)
  
  ip <- installed.packages()[, "Package"]
  if (! pkgName %in% ip) {
    if (is.null(repos) && isBioconductor) {
      InternetTest <- try(readLines("http://www.bioconductor.org"), silent=TRUE)
    } else if (is.null(repos)) {
      InternetTest <- try(readLines("http://cran.r-project.org"), silent=TRUE)
    } else {
      InternetTest <- try(readLines(repos), silent=TRUE)
    } 
    if (inherits(InternetTest,"try-error")) {
      return(FALSE)
    } else {
      if (is.null(repos) && isBioconductor) {
        source("http://www.bioconductor.org/biocLite.R")
        biocLite(pkgName, suppressUpdates = suppressUpdates, ask=FALSE, type=bioctype)
      } else if (is.null(repos)) {
        #fixed problem of downloading XML R package.
        local({r <- getOption("repos"); r["CRAN"] <- "http://cran.r-project.org"; options(repos=r)}) 
        install.packages(pkgName, repos=getOption("repos"))
      } else {
        # Allow to install package from r-forge. For example, Cairo.
        install.packages(pkgName, repos=repos, type=type)
      }
      #load again after installing
      pkg.exists <- library(pkgName, character.only=TRUE, logical.return=TRUE)
      if (!pkg.exists) return(FALSE)
    }
  } else {
    library(pkgName, character.only=TRUE)    
  }
  options(warn=curwarn)
  return(TRUE)
}

AverageDuplic <- function(mat, duplic, pheno, id=NULL,RemoveUnpaired = FALSE,Block = NULL) {
  # The function is called by WriteDataToFiles() in <WriteDataToFiles.R> only.
  # Input:
  #   mat: a matrix
  #   duplic=a string of characters
  #   pheno=a string of characters
  #   Block=a string of characters or NULL
  # Assume CheckPheno(duplic,pheno)==TRUE
  # Example: 
  #   duplic <- c('4','4','2','2','3','3','3','5','5')
  #   pheno <- c('a','a','b','b','b','b','b','a','a')
  #   mat <- matrix(1:45, nr=5); mat[1,1] <- NA
  #   tmp <- AverageDuplic(mat,duplic,pheno);tmp
  
  if (is.null(id))  id <- duplic
  nc <- length(unique(duplic))
  newmat <- matrix(0, nr=nrow(mat), nc=nc)
  newpheno <- rep(0, nc)
  HasBlock <- !is.null(Block)
  newBlock <- NULL
  if(HasBlock) newBlock <- rep(0, nc)
  newid <- rep(0, nc)
  j <- 0
  for(i in sort(unique(duplic)))   {
    ind <- which(duplic==i)
    if (length(ind)==1) {
      tmp <- mat[,ind]
    } else {
      tmp <- apply(mat[,ind],1,mean, na.rm=TRUE)
    }
    j <- j+1
    newmat[,j] <- tmp
    newpheno[j] <- pheno[ind[1]]
    if(HasBlock) newBlock[j] <- Block[ind[1]]
    newid[j] <- id[ind[1]]
  }
  if(RemoveUnpaired) {
    code <- rep(TRUE,length(newid))
    for(i in 1:length(newid) ) {
      code[i] <- (length(newid[newid == newid[i] ]) == 2) &
        (length(unique(newpheno[newid == newid[i] ])) == 2 )
    }
    newmat   <- newmat[,code]
    newid    <- newid[code]
    newpheno <- newpheno[code]
  }
  dimnames(newmat) <- NULL
  return(list(mat=newmat, pheno=newpheno, id=newid,block = newBlock))  
}

CheckPheno <- function(duplic, pheno) 
  nrow(unique(data.frame(duplic,pheno))) == length(unique(duplic))

