# Test Calculation function ---------------------------------------------------

#' Test classComparison() function
#'
#' This function will load a test dataset to run the \code{classComparison} function.
#'
#' @param dataset character string specifying one of "Brca", "Perou" or "Pomeroy" datasets.
#' @param projectPath characteer string specifying the project path. Default is C:/Users/UserName/Documents/$dataset.
#' @param outputName character string for the output folder name.
#' @param generateHTML logical. If \code{TRUE}, show an HTML file with detailed class comparison results in the system default browser.
#' @return A list as returned by \code{classComparison}.
#' @export
#' @seealso \code{\link{classComparison}}
#' @examples
#' test.classComparison("Brca")
#' 
test.classComparison <- function(dataset = c("Brca", "Perou", "Pomeroy"), 
                                  projectPath, 
                                  outputName = "ClassComparison",
                                  generateHTML = FALSE) {
  dataset <- match.arg(dataset)
  if (missing(projectPath))  projectPath <- file.path(Sys.getenv("R_USER"), dataset)
  # reading data
  geneIds <- read.delim(system.file("extdata", paste0(dataset, "_GENEID.txt"), package = "classComparison"), as.is = TRUE, colClasses = "character")
  singleChannel <- ifelse(dataset == "Pomeroy", TRUE, FALSE)
  if (singleChannel) {
    x <- read.delim(system.file("extdata", paste0(dataset, "_NORMALIZEDLOGINTENSITY.TXT"), package = "classComparison"), header = FALSE)
  } else {
    x <- read.delim(system.file("extdata", paste0(dataset, "_LOGRAT.TXT"), package = "classComparison"), header = FALSE)
  }
  geneFilter <- scan(system.file("extdata", paste0(dataset, "_FILTER.TXT"), package = "classComparison"), quiet = TRUE)
  expDesign <- read.delim(system.file("extdata", paste0(dataset, "_EXPDESIGN.txt"), package = "classComparison"), as.is = TRUE)
  
  # For simplicity, I assume the samples are ordered as the class labels.
  arrayIds <- expDesign[, 1]
  exprData <- x
  colnames(exprData) <- expDesign[, 1]
  
  # Pick the 3rd column as the class variable.
  nColumn = 3
  ClassVariableName = gsub("[.]"," ",colnames(expDesign)[nColumn])
  ClassVariableValues <- expDesign[, nColumn]
  
  # Use the 2nd column as the pairing variable for Perou.
  isPaired = FALSE
  if (isPaired) {
    nColumn = 2
    GroupVariable <- expDesign[, nColumn]
  } else {
    GroupVariable <- NULL
  }
  
  
  # Use the 2nd column as the block variable for Perou.
  HasBlock = FALSE
  if (HasBlock) {
    nColumn = 2
    BlockVariable <- expDesign[, nColumn]
    BlockVariableName <- colnames(expDesign)[nColumn]
  } else {
    BlockVariable <- NULL
    BlockVariableName <-NULL
  }
  
  # Replicates testing for Perou.
  HasDuplic = FALSE
  if (HasDuplic) {
    duplic <- c("1","2","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","32","33","34","35","36","35","36")
  } else {
    duplic <- NULL
  }
  
  resList <- classComparison(exprData=exprData,
                             geneIds=geneIds,
                             ClassVariableName=ClassVariableName,
                             ClassVariableValues=ClassVariableValues,
                             geneFilter=geneFilter,
                             IsSingleChannel=singleChannel,
                             isPaired=isPaired,
                             GroupVariable=GroupVariable,
                             HasBlock=HasBlock,
                             BlockVariableName=BlockVariableName,
                             BlockVariable=BlockVariable,
                             HasDuplic=HasDuplic,
                             duplic=duplic,
                             ProportionOp="optUnivariateTest",
                             #"optUnivariateTest":  Univariate tests.
                             #"optProportion":  Multivariate permutation tests.
                             #"optLocalFDR":  Local False Discovery Rate tests.
                             SignifLFDR=0.5,
                             DoRandomVariance=TRUE,
                             DoUnivariatePermTest=FALSE,
                             GlobalTest=FALSE,
                             DoFoldThresh=FALSE,
                             projectPath=projectPath,
                             outputName=outputName,
                             generateHTML=generateHTML)
  
  return(resList)
}
