% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classComparison.R
\name{classComparison}
\alias{classComparison}
\title{Class Comparison between Groups of Arrays}
\usage{
classComparison(exprData, geneIds, ClassVariableName, ClassVariableValues,
  geneFilter, IsSingleChannel, isPaired = FALSE, HasBlock = FALSE,
  HasDuplic = FALSE, GroupVariable = NULL, BlockVariable = NULL,
  duplic = NULL, BlockVariableName = NULL,
  ProportionOp = "optUnivariateTest", UnivarSignifLevel = 0.001,
  MaxProportionFD = 0.1, ConfLevel = 80, SignifLFDR = 0.05,
  DoRandomVariance = TRUE, DoUnivariatePermTest = FALSE,
  NumPermUnivariate = 10000, GlobalTest = FALSE,
  NumPermMultivariate = 1000, DoFoldThresh = FALSE,
  NumFoldThreshold = 2, NumAlphaPairwise = 0.01,
  IngenuityRefClass = "Automatically create the combination output",
  projectPath, outputName = "ClassComparison", generateHTML = FALSE)
}
\arguments{
\item{exprData}{matrix of gene expression data for samples. Rows are genes and columns are arrays. Its column names must be provided.}

\item{geneIds}{matrix/data frame of gene identifiers, such as Gene Symbols, Unique Ids, Clone Ids, Entrez Ids.}

\item{ClassVariableName}{character string of the sample class variable name.}

\item{ClassVariableValues}{vector of sample classes.}

\item{geneFilter}{vector of 1/0's of the same length as genes. 1 means to keep the gene while 0 means to exclude the gene.}

\item{IsSingleChannel}{logical. If \code{TRUE}, the data are single channel data, such as Affymatrix data.}

\item{isPaired}{logical. If \code{TRUE}, samples are paired.}

\item{HasBlock}{logical. If \code{TRUE}, samples have block variables.}

\item{HasDuplic}{logical. If \code{TRUE}, samples need to average over replicate variables.}

\item{GroupVariable}{vector of pairing variables for samples.}

\item{BlockVariable}{vector of block variables for samples.}

\item{duplic}{vector of average over repicate variables for samples.}

\item{BlockVariableName}{character string of the block variable name.}

\item{ProportionOp}{character string of the method used to find gene lists.
\itemize{
  \item "optUnivariateTest":  Univariate tests.
  \item "optProportion":  Multivariate permutation tests.
  \item "optLocalFDR":  Local False Discovery Rate tests.
}}

\item{UnivarSignifLevel}{numeric for a significance level of univariate tests. Default is 0.001.}

\item{MaxProportionFD}{numeric for maximum proportion of false discoveries. Default is 0.1.}

\item{ConfLevel}{numeric for multivariate permutation tests, between 0 and 100 percent. Default is 80.}

\item{SignifLFDR}{numeric for a significance level of local False discovery rate tests. Default is 0.05.}

\item{DoRandomVariance}{logical. If \code{TRUE}, use the random variance model for univariate tests. Default is \code{TRUE}.}

\item{DoUnivariatePermTest}{logical. If \code{TRUE}, perform univariate permutation tests. Default is \code{FALSE}.}

\item{NumPermUnivariate}{numeric specifying the number of permutations for the univariate permutation tests. Default is 10000.}

\item{GlobalTest}{logical. If \code{TRUE}, perform P-value for global tests. Default is \code{FALSE}.}

\item{NumPermMultivariate}{numeric specifying the number of permutations for the multivariate tests. Default is 1000.}

\item{DoFoldThresh}{logical. If \code{TRUE}, restrct gene list based on fold change threshold. Default is \code{FALSE}.}

\item{NumFoldThreshold}{numeric specifying the fold change threshold. Default is 2.}

\item{NumAlphaPairwise}{numeric specifying the P-value threshold for pairwise difference. Default is 0.01.}

\item{IngenuityRefClass}{character string of the reference class name for Ingenuity output. Default is "Automatically create the combination output".}

\item{projectPath}{character string specifying the full project path.}

\item{outputName}{character string specifying the output file folder name. The folder is in the \code{projectPath}. Default is "ClassComparison".}

\item{generateHTML}{logical. If \code{TRUE}, show an HTML file with detailed class comparison results in the system default browser.}
}
\value{
A list includes the following objects:
\itemize{
   \item \code{classifierTable}: a data frame with the performance of classifiers.
   \item \code{workPath}: the path for Fortran and other intermediate output files.
   \item \code{outputPath}: the path for final result output files.
}
}
\description{
This function is used for comparing two or more pre-defined classes. 
The classes to be compared are in a vector. The vector can be any set 
of numerical, character or character string data. If an entry for 
a particular sample is left blank in the vector, that sample will be 
omitted from the class comparison analysis.
}
\details{
The Class Comparison between Groups of Arrays function computes a t-test or 
F-test separately for each gene using the normalized log-ratios for cDNA arrays 
and the normalized log-intensities for one color oligonucleotide arrays. The 
F-test is a generalization of the two-sample t-test for comparing values among 
groups. It has the option of using the random variance version of the t-test 
or F-test. They provide for sharing information among genes of the within-class 
variance in log-ratios or log signals. The class comparison function computes 
the number of genes that are differentially expressed among the classes at the 
statistical significance level selected in the F-test and creates a gene list 
containing information about the significant genes. 
 
It implements the Class Comparison between Groups of Arrays tool in BRB-ArrayTools.

Please see the BRB-ArrayTools manual (\url{https://brb.nci.nih.gov/BRB-ArrayTools/Documentation.html}) for details.
}
\note{
Following functions were disabled:
 \itemize{
   \item Create the chromosome distribution plot.
   \item Perform GO Observed vs Expected analysis.
 }
}
\examples{

dataset<-"Brca"
# Gene IDs
geneIds <- read.delim(system.file("extdata", paste0(dataset, "_GENEID.txt")
  , package = "classComparison"), as.is = TRUE, colClasses = "character") 
# Expression data, and here are log ratio.
x <- read.delim(system.file("extdata", paste0(dataset, "_LOGRAT.TXT")
  , package = "classComparison"), header = FALSE)
# Gene filter information, 1 - pass the filter, 0 - filtered
geneFilter <- scan(system.file("extdata", paste0(dataset, "_FILTER.TXT")
  , package = "classComparison"), quiet = TRUE)
# Class information
expDesign <- read.delim(system.file("extdata", paste0(dataset, "_EXPDESIGN.txt")
  , package = "classComparison"), as.is = TRUE)
# Pick the first column as the array IDs.
arrayIds <- expDesign[, 1]
exprData <- x
colnames(exprData) <- expDesign[, 1]
# Pick the 3rd column as the class variable.
nColumn = 3
ClassVariableName = gsub("[.]"," ",colnames(expDesign)[nColumn])
ClassVariableValues <- expDesign[, nColumn]
projectPath <- tempdir()
outputName = "ClassComparison"
singleChannel <- ifelse(dataset == "Pomeroy", TRUE, FALSE)
generateHTML = TRUE
resList <- classComparison(exprData=exprData,
                             geneIds=geneIds,
                             ClassVariableName=ClassVariableName,
                             ClassVariableValues=ClassVariableValues,
                             geneFilter=geneFilter,
                             IsSingleChannel=singleChannel,
                             projectPath=projectPath,
                             outputName=outputName,
                             generateHTML=generateHTML)

}
\seealso{
\code{\link{test.classComparison}}
}
